@@ -2,7 +2,7 @@
    Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
 
    Written by:  Andrew Kachites McCallum <mccallum@gnu.ai.mit.edu>
-   Rewrite by:  Richard Frith-Macdonald <rfm@gnu.org>
+   Written by:  Richard Frith-Macdonald <rfm@gnu.org>
    Date: March 1995
    
    This file is part of the GNUstep Base Library.
@@ -40,6 +40,7 @@ extern "C" {
 
 #if	OS_API_VERSION(GS_API_OSSPEC,GS_API_LATEST)
 
+GS_EXPORT_CLASS
 @interface NSArchiver : NSCoder
 {
 #if	GS_EXPOSE(NSArchiver)
@@ -157,8 +158,9 @@ extern "C" {
 @end
 #endif
 
-
 
+
+GS_EXPORT_CLASS
 @interface NSUnarchiver : NSCoder
 {
 #if	GS_EXPOSE(NSUnarchiver)
