@@ -11,12 +11,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
   */
 
 #ifndef __NSCalendarDate_h_GNUSTEP_BASE_INCLUDE
@@ -32,11 +32,14 @@ extern "C" {
 @class	NSTimeZone;
 @class	NSTimeZoneDetail;
 
+GS_EXPORT_CLASS
 @interface NSCalendarDate : NSDate
 {
+#if	GS_EXPOSE(NSCalendarDate)
   NSTimeInterval	_seconds_since_ref;
   NSString		*_calendar_format;
   NSTimeZone		*_time_zone;
+#endif
 }
 
 // Getting an NSCalendar Date
@@ -104,7 +107,7 @@ extern "C" {
 - (NSString*) descriptionWithCalendarFormat: (NSString*)format;
 - (NSString*) descriptionWithCalendarFormat: (NSString*)format
 				     locale: (NSDictionary*)locale;
-- (NSString*) descriptionWithLocale: (NSDictionary*)locale;
+- (NSString*) descriptionWithLocale: (id)locale;
 
 // Getting and Setting Calendar Formats
 - (NSString*) calendarFormat;
@@ -123,27 +126,12 @@ extern "C" {
 
 #if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
-/**
- *  Adds <code>-weekOfYear</code> method.
- */
-@interface NSCalendarDate (GSCategories)
-/**
- * The ISO standard week of the year is based on the first week of the
- * year being that week (starting on monday) for which the thursday
- * is on or after the first of january.<br />
- * This has the effect that, if january first is a friday, saturday or
- * sunday, the days of that week (up to and including the sunday) are
- * considered to be in week 53 of the preceding year. Similarly if the
- * last day of the year is a monday tuesday or wednesday, these days are
- * part of week 1 of the next year.
- */
-- (NSInteger) weekOfYear;
-@end
-
 @interface NSCalendarDate (GregorianDate)
 
 - (NSInteger) lastDayOfGregorianMonth: (NSInteger)month year: (NSInteger)year;
-- (NSInteger) absoluteGregorianDay: (NSInteger)day month: (NSInteger)month year: (NSInteger)year;
+- (NSInteger) absoluteGregorianDay: (NSInteger)day
+			     month: (NSInteger)month
+			      year: (NSInteger)year;
 - (void) gregorianDateFromAbsolute: (NSInteger)d
 			       day: (NSInteger*)day
 			     month: (NSInteger*)month
@@ -173,6 +161,10 @@ extern "C" {
 @end
 #endif
 
+#if     !NO_GNUSTEP && !defined(GNUSTEP_BASE_INTERNAL)
+#import <GNUstepBase/NSCalendarDate+GNUstepBase.h>
+#endif
+
 #if	defined(__cplusplus)
 }
 #endif
