@@ -1,5 +1,5 @@
 /* Interface to debugging utilities for GNUStep and OpenStep
-   Copyright (C) 1997,1999 Free Software Foundation, Inc.
+   Copyright (C) 1997-2020 Free Software Foundation, Inc.
 
    Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Date: August 1997
@@ -44,6 +44,35 @@
 extern "C" {
 #endif
 
+/** Protocol for a delegate, set as an extension in some classes, to handle
+ * debug logging of low level I/O.  The rationale for this protocol is that
+ * on occasion debug logging may be required, but the data being logged may
+ * contain sensitive information which should not be writtent to file.  In
+ * that situation, the delegate may filter/mask the sensitive information
+ * from the logs by taking over the simpel writing to stderr that the inbuilt
+ * debug logging provides.
+ */ 
+@protocol GSLogDelegate <NSObject>
+/** Method sent to the delegate to ask it to log a chunk of data that
+ * has been read.  The delegate should return YES if it has handled
+ * the logging, NO if it wants the default mechanism to be used.<br />
+ * The handle is the object which is performing the read operation.
+ */
+- (BOOL) getBytes: (const uint8_t*)bytes
+         ofLength: (NSUInteger)length
+         byHandle: (NSObject*)handle;
+
+/** Method sent to the delegate to ask it to log a chunk of data that
+ * has been written (or is immediately going to be written).
+ * The delegate should return YES if it has handled the logging,
+ * NO if it wants the default logging mechanism to be used.<br />
+ * The handle is the object which is performing the write operation.
+ */
+- (BOOL) putBytes: (const uint8_t*)bytes
+         ofLength: (NSUInteger)length
+         byHandle: (NSObject*)handle;
+@end
+
 /*
  *	Functions for debugging object allocation/deallocation
  *
@@ -75,6 +104,13 @@ extern "C" {
  *  GSDebugAllocationListRecordedObjects() 
  *  GSDebugAllocationTagRecordedObject()
  */
+
+/**
+ * A global variable that you can use to enable debug behaviour in
+ * your program.
+ */
+extern BOOL NSDebugEnabled;
+
 #ifndef	NDEBUG
 
 /**
