@@ -1,4 +1,4 @@
-/* Interface for GNU Objective-C version of NSDistantObject
+/**Interface for GNU Objective-C version of NSDistantObject
    Copyright (C) 1997 Free Software Foundation, Inc.
    
    Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
@@ -15,12 +15,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */
 
 #ifndef __NSDistantObject_h_GNUSTEP_BASE_INCLUDE
@@ -35,6 +35,7 @@ extern "C" {
 
 @class	NSConnection;
 
+GS_EXPORT_CLASS
 @interface NSDistantObject : NSProxy <NSCoding>
 {
 #if	GS_EXPOSE(NSDistantObject)
