@@ -1,7 +1,7 @@
 /**Interface for NSMetadataQuery for GNUStep
    Copyright (C) 2012 Free Software Foundation, Inc.
 
-   Written by: Gregory Casamento
+   Written by: Gregory John Casamento <greg.casamento@gmail.com>
    Date: 2012
    
    This file is part of the GNUstep Base Library.
@@ -20,8 +20,6 @@
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110 USA.
-
-   AutogsdocSource: NSMetadataQuery.h
 */ 
 
 #ifndef __NSMetadata_h_GNUSTEP_BASE_INCLUDE
