@@ -15,12 +15,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
 
   AutogsdocSource: NSNotification.m
   AutogsdocSource: NSNotificationCenter.m
@@ -43,6 +43,9 @@ extern "C" {
 @class NSLock;
 @class NSOperationQueue;
 
+typedef NSString* NSNotificationName;
+
+GS_EXPORT_CLASS
 @interface NSNotification : NSObject <NSCopying, NSCoding>
 
 /* Creating a Notification Object */
@@ -66,11 +69,12 @@ extern "C" {
 DEFINE_BLOCK_TYPE(GSNotificationBlock, void, NSNotification *);
 #endif
 
+GS_EXPORT_CLASS
 @interface NSNotificationCenter : NSObject
 {
 #if	GS_EXPOSE(NSNotificationCenter)
 @private
-  void	*_table;
+  void	         *_table;
 #endif
 }
 
