@@ -1,5 +1,4 @@
-
-/* Definition of class NSScriptCoercionHandler
+/**Definition of class NSScriptCoercionHandler
    Copyright (C) 2019 Free Software Foundation, Inc.
    
    By: Gregory John Casamento <greg.casamento@gmail.com>
@@ -34,6 +33,7 @@ extern "C" {
 
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_0, GS_API_LATEST)
 
+GS_EXPORT_CLASS
 @interface NSScriptCoercionHandler : NSObject
 
 @end
