@@ -53,6 +53,7 @@ GS_EXPORT NSString *const NSGrammarRange;
 GS_EXPORT NSString *const NSGrammarUserDescription;
 #endif
 
+GS_EXPORT_CLASS
 @interface NSSpellServer : NSObject
 {
 #if	GS_EXPOSE(NSSpellServer)
@@ -162,6 +163,12 @@ findMisspelledWordInString: (NSString *)stringToCheck
 
 @end
 
+#if	!NO_GNUSTEP
+// Function to create name for spell server
+GS_EXPORT NSString*
+GSSpellServerName(NSString *vendor, NSString *language);
+#endif
+
 #if     defined(__cplusplus)
 }
 #endif
