@@ -1,4 +1,4 @@
-/* Interface for NSURLDownload for GNUstep
+/**Interface for NSURLDownload for GNUstep
    Copyright (C) 2006 Software Foundation, Inc.
 
    Written by:  Richard Frith-Macdonald <frm@gnu.org>
@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSURLDownload_h_GNUSTEP_BASE_INCLUDE
@@ -44,6 +44,7 @@ extern "C" {
 /**
  * Handles download to file.
  */
+GS_EXPORT_CLASS
 @interface NSURLDownload : NSObject
 {
 #if	GS_EXPOSE(NSURLDownload)
@@ -121,6 +122,7 @@ extern "C" {
  */
 
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_7, GS_API_LATEST)
+/** <ignore> autogsdoc will ignore everything up to the end tag */
 @protocol NSURLDownloadDelegate <NSObject>
 #if GS_PROTOCOLS_HAVE_OPTIONAL
 @optional
@@ -128,6 +130,7 @@ extern "C" {
 @end
 @interface NSObject (NSURLDownloadDelegate)
 #endif
+/** end of text ignored by autogsdoc </ignore> */
 #else
 @interface NSObject (NSURLDownloadDelegate)
 #endif
