@@ -1,4 +1,4 @@
-/* Interface for NSURLProtocol for GNUstep
+/**Interface for NSURLProtocol for GNUstep
    Copyright (C) 2006 Software Foundation, Inc.
 
    Written by:  Richard Frith-Macdonald <frm@gnu.org>
@@ -14,19 +14,19 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSURLProtocol_h_GNUSTEP_BASE_INCLUDE
 #define __NSURLProtocol_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -44,6 +44,7 @@ extern "C" {
 @class NSURLProtocol;
 @class NSURLRequest;
 @class NSURLResponse;
+@class NSURLSessionTask;
 
 
 /**
@@ -113,9 +114,10 @@ extern "C" {
  * structure for the subclasses.
  * </p>
  * <p>You never instantiate NSURLProtocol yourself ... it should only
- * ever be done by other classes within mthe URL loading system.
+ * ever be done by other classes within the URL loading system.
  * </p>
  */
+GS_EXPORT_CLASS
 @interface NSURLProtocol : NSObject
 {
 #if	GS_EXPOSE(NSURLProtocol)
@@ -171,17 +173,27 @@ extern "C" {
  * The cachedResponse may be the result of a previous load of the
  * request (in which case the protocol may validate and use it).<br />
  * The client is the object which receives messages about the progress
- * of the load.
+ * of the load.  This is retained by the protocl instance and is released
+ * once the last message has been sent to it.
  */
 - (id) initWithRequest: (NSURLRequest *)request
 	cachedResponse: (NSCachedURLResponse *)cachedResponse
 		client: (id <NSURLProtocolClient>)client;
 
+- (instancetype) initWithTask: (NSURLSessionTask*)task 
+               cachedResponse: (NSCachedURLResponse*)cachedResponse 
+                       client: (id<NSURLProtocolClient>)client;
+
 /**
  * Returns the request handled by the receiver.
  */
 - (NSURLRequest *) request;
 
+/**
+ * Returns the task handled by the receiver.
+ */
+- (NSURLSessionTask *) task;
+
 @end
 
 /**
@@ -197,6 +209,11 @@ extern "C" {
  */
 + (BOOL) canInitWithRequest: (NSURLRequest *)request;
 
+/** This method is called to decide whether a class can deal with
+ * the specified task. The abstract class implementation return NO.
+ */
++ (BOOL) canInitWithTask: (NSURLSessionTask*)task;
+
 /** <override-subclass />
  * Returns the 'canonical' version of the request.<br />
  * The canonical form is used to look up requests in the cache by
