@@ -1,4 +1,4 @@
-/* Interface for NSXMLDTDNode for GNUStep
+/**Interface for NSXMLDTDNode for GNUStep
    Copyright (C) 2008 Free Software Foundation, Inc.
 
    Written by:  Richard Frith-Macdonald <rfm@gnu.org>
@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
 */
 
 #ifndef __NSXMLDTDNode_h_GNUSTEP_BASE_INCLUDE
@@ -73,11 +73,12 @@ typedef NSUInteger NSXMLDTDNodeKind;
  * <desc>nil</desc>
  * </deflist>
  */
+GS_EXPORT_CLASS
 @interface NSXMLDTDNode : NSXMLNode
 {
 #if     GS_NONFRAGILE
 #  if	defined(GS_NSXMLDTDNode_IVARS)
-@public GS_NSXMLDTDNode_IVARS
+@public GS_NSXMLDTDNode_IVARS;
 #  endif
 #endif
   /* The pointer to private additional data used to avoid breaking ABI
@@ -115,7 +116,7 @@ typedef NSUInteger NSXMLDTDNodeKind;
 /**
  * Sets what kind of DTD node this is.
  */
-- (void) setDTDKind: (NSXMLDTDNodeKind)kind;
+- (void) setDTDKind: (NSXMLDTDNodeKind)nodeKind;
 
 /**
  * Sets the notation name if the receiver is an entity.
