@@ -30,11 +30,13 @@
 #if	defined(GNUSTEP_BASE_INTERNAL)
 #import "Foundation/NSObject.h"
 #import "Foundation/NSEnumerator.h"
+#import "Foundation/NSException.h"
 #import "Foundation/NSGarbageCollector.h"
 #import "Foundation/NSZone.h"
 #else
 #import <Foundation/NSObject.h>
 #import <Foundation/NSEnumerator.h>
+#import <Foundation/NSException.h>
 #import <Foundation/NSGarbageCollector.h>
 #import <Foundation/NSZone.h>
 #endif
@@ -44,10 +46,11 @@ extern "C" {
 #endif
 
 /* To easily un-inline functions for debugging */
-#ifndef	INLINE
-#define INLINE inline
+#ifndef GS_STATIC_INLINE
+#define GS_STATIC_INLINE static inline
 #endif
 
+
 /*
  *      NB. This file is intended for internal use by the GNUstep libraries
  *      and may change siugnificantly between releases.
@@ -154,9 +157,12 @@ extern "C" {
 #  define GSI_MAP_WRITE_VAL(M, addr, obj) (*(addr) = obj)
 #endif
 #if	GSI_MAP_HAS_VALUE
-#define GSI_MAP_NODE_IS_EMPTY(M, node) (((GSI_MAP_READ_VALUE(M, &node->key).addr) == 0) || ((GSI_MAP_READ_VALUE(M, &node->value).addr == 0)))
+#define GSI_MAP_NODE_IS_EMPTY(M, node) \
+  (((GSI_MAP_READ_KEY(M, &node->key).addr) == 0) \
+  || ((GSI_MAP_READ_VALUE(M, &node->value).addr == 0)))
 #else
-#define GSI_MAP_NODE_IS_EMPTY(M, node) (((GSI_MAP_READ_VALUE(M, &node->key).addr) == 0))
+#define GSI_MAP_NODE_IS_EMPTY(M, node) \
+  (((GSI_MAP_READ_KEY(M, &node->key).addr) == 0))
 #endif
 
 /*
@@ -372,6 +378,11 @@ struct	_GSIMapTable {
   GSI_MAP_EXTRA	extra;
 #endif
 };
+#define GSI_MAP_TABLE_T GSIMapTable_t
+#endif
+
+#ifndef GSI_MAP_TABLE_S
+#define GSI_MAP_TABLE_S sizeof(GSI_MAP_TABLE_T)
 #endif
 
 typedef struct	_GSIMapEnumerator {
@@ -387,27 +398,27 @@ typedef struct _GSIMapEnumerator GSIMapEnumerator_t;
 #endif
 typedef GSIMapEnumerator_t	*GSIMapEnumerator;
 
-static INLINE GSIMapBucket
+GS_STATIC_INLINE GSIMapBucket
 GSIMapPickBucket(unsigned hash, GSIMapBucket buckets, uintptr_t bucketCount)
 {
   return buckets + hash % bucketCount;
 }
 
-static INLINE GSIMapBucket
+GS_STATIC_INLINE GSIMapBucket
 GSIMapBucketForKey(GSIMapTable map, GSIMapKey key)
 {
   return GSIMapPickBucket(GSI_MAP_HASH(map, key),
     map->buckets, map->bucketCount);
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapLinkNodeIntoBucket(GSIMapBucket bucket, GSIMapNode node)
 {
   node->nextInBucket = bucket->firstNode;
   bucket->firstNode = node;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapUnlinkNodeFromBucket(GSIMapBucket bucket, GSIMapNode node)
 {
   if (node == bucket->firstNode)
@@ -427,38 +438,38 @@ GSIMapUnlinkNodeFromBucket(GSIMapBucket bucket, GSIMapNode node)
   node->nextInBucket = 0;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapAddNodeToBucket(GSIMapBucket bucket, GSIMapNode node)
 {
   GSIMapLinkNodeIntoBucket(bucket, node);
   bucket->nodeCount += 1;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapAddNodeToMap(GSIMapTable map, GSIMapNode node)
 {
   GSIMapBucket	bucket;
 
-  bucket = GSIMapBucketForKey(map, node->key);
+  bucket = GSIMapBucketForKey(map, GSI_MAP_READ_KEY(map, &node->key));
   GSIMapAddNodeToBucket(bucket, node);
   map->nodeCount++;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRemoveNodeFromBucket(GSIMapBucket bucket, GSIMapNode node)
 {
   bucket->nodeCount--;
   GSIMapUnlinkNodeFromBucket(bucket, node);
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRemoveNodeFromMap(GSIMapTable map, GSIMapBucket bkt, GSIMapNode node)
 {
   map->nodeCount--;
   GSIMapRemoveNodeFromBucket(bkt, node);
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapFreeNode(GSIMapTable map, GSIMapNode node)
 {
   GSI_MAP_RELEASE_KEY(map, node->key);
@@ -472,7 +483,7 @@ GSIMapFreeNode(GSIMapTable map, GSIMapNode node)
   map->freeNodes = node;
 }
 
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapRemoveAndFreeNode(GSIMapTable map, uintptr_t bkt, GSIMapNode node)
 {
   GSIMapNode	next = node->nextInBucket;
@@ -481,13 +492,14 @@ GSIMapRemoveAndFreeNode(GSIMapTable map, uintptr_t bkt, GSIMapNode node)
   return next;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRemoveWeak(GSIMapTable map)
 {
-  uintptr_t bucketCount = map->bucketCount;
-  GSIMapBucket bucket = map->buckets;
   if (GSI_MAP_ZEROED(map))
     {
+      uintptr_t bucketCount = map->bucketCount;
+      GSIMapBucket bucket = map->buckets;
+
       while (bucketCount-- > 0)
         {
           GSIMapNode node = bucket->firstNode;
@@ -508,7 +520,7 @@ GSIMapRemoveWeak(GSIMapTable map)
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRemangleBuckets(GSIMapTable map,
   GSIMapBucket old_buckets, uintptr_t old_bucketCount,
   GSIMapBucket new_buckets, uintptr_t new_bucketCount)
@@ -531,8 +543,9 @@ GSIMapRemangleBuckets(GSIMapTable map,
 		  GSIMapBucket	bkt;
 
 		  GSIMapRemoveNodeFromBucket(old_buckets, node);
-		  bkt = GSIMapPickBucket(GSI_MAP_HASH(map, node->key),
-		    new_buckets, new_bucketCount);
+		  bkt = GSIMapPickBucket(GSI_MAP_HASH(map,
+		    GSI_MAP_READ_KEY(map, &node->key)),
+		      new_buckets, new_bucketCount);
 		  GSIMapAddNodeToBucket(bkt, node);
 		}
 	    }
@@ -549,28 +562,22 @@ GSIMapRemangleBuckets(GSIMapTable map,
 	  GSIMapBucket	bkt;
 
 	  GSIMapRemoveNodeFromBucket(old_buckets, node);
-	  bkt = GSIMapPickBucket(GSI_MAP_HASH(map, node->key),
-	    new_buckets, new_bucketCount);
+	  bkt = GSIMapPickBucket(GSI_MAP_HASH(map,
+	    GSI_MAP_READ_KEY(map, &node->key)),
+	      new_buckets, new_bucketCount);
 	  GSIMapAddNodeToBucket(bkt, node);
 	}
       old_buckets++;
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapMoreNodes(GSIMapTable map, unsigned required)
 {
   GSIMapNode	*newArray;
-  uintptr_t	arraySize = (map->chunkCount+1)*sizeof(GSIMapNode);
 
-#if     GS_WITH_GC
-  /* We don't want our nodes collected before we have finished with them,
-   * so we must keep the array of pointers to memory chunks in scanned memory.
-   */
-  newArray = (GSIMapNode*)NSAllocateCollectable(arraySize, NSScannedOption);
-#else
-  newArray = (GSIMapNode*)NSZoneMalloc(map->zone, arraySize);
-#endif
+  newArray = (GSIMapNode*)NSZoneCalloc(map->zone,
+    (map->chunkCount+1), sizeof(GSIMapNode));
   if (newArray)
     {
       GSIMapNode	newNodes;
@@ -580,9 +587,7 @@ GSIMapMoreNodes(GSIMapTable map, unsigned required)
 	{
 	  memcpy(newArray, map->nodeChunks,
 	    (map->chunkCount)*sizeof(GSIMapNode));
-#if     !GS_WITH_GC
 	  NSZoneFree(map->zone, map->nodeChunks);
-#endif
 	}
       map->nodeChunks = newArray;
 
@@ -601,12 +606,8 @@ GSIMapMoreNodes(GSIMapTable map, unsigned required)
 	{
 	  chunkCount = required;
 	}
-#if     GS_WITH_GC
-      newNodes = GSI_MAP_NODES(map, chunkCount);
-#else
       newNodes
-        = (GSIMapNode)NSZoneMalloc(map->zone, chunkCount*sizeof(GSIMapNode_t));
-#endif
+        = (GSIMapNode)NSZoneCalloc(map->zone, chunkCount, sizeof(GSIMapNode_t));
       if (newNodes)
 	{
 	  map->nodeChunks[map->chunkCount++] = newNodes;
@@ -617,17 +618,26 @@ GSIMapMoreNodes(GSIMapTable map, unsigned required)
 	    }
 	  map->freeNodes = newNodes;
 	}
+      else
+	{
+	  [NSException raise: NSMallocException format: @"No memory for nodes"];
+	}
+    }
+  else
+    {
+      [NSException raise: NSMallocException format: @"No memory for chunks"];
     }
 }
 
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapNodeForKeyInBucket(GSIMapTable map, GSIMapBucket bucket, GSIMapKey key)
 {
   GSIMapNode	node = bucket->firstNode;
 
   if (GSI_MAP_ZEROED(map))
     {
-      while ((node != 0) && GSI_MAP_EQUAL(map, GSI_MAP_READ_KEY(map, &node->key), key) == NO)
+      while ((node != 0)
+	&& GSI_MAP_EQUAL(map, GSI_MAP_READ_KEY(map, &node->key), key) == NO)
 	{
 	  GSIMapNode	tmp = node->nextInBucket;
 
@@ -640,14 +650,15 @@ GSIMapNodeForKeyInBucket(GSIMapTable map, GSIMapBucket bucket, GSIMapKey key)
 	}
       return node;
     }
-  while ((node != 0) && GSI_MAP_EQUAL(map, GSI_MAP_READ_KEY(map, &node->key), key) == NO)
+  while ((node != 0)
+    && GSI_MAP_EQUAL(map, GSI_MAP_READ_KEY(map, &node->key), key) == NO)
     {
       node = node->nextInBucket;
     }
   return node;
 }
 
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapNodeForKey(GSIMapTable map, GSIMapKey key)
 {
   GSIMapBucket	bucket;
@@ -662,7 +673,7 @@ GSIMapNodeForKey(GSIMapTable map, GSIMapKey key)
   return node;
 }
 
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapFirstNode(GSIMapTable map)
 {
   if (map->nodeCount > 0)
@@ -711,7 +722,7 @@ GSIMapFirstNode(GSIMapTable map)
  * or pointer values that are their own hash values (when converted to unsigned
  * integers) and can be compared with a test for integer equality.
  */
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
 {
   GSIMapBucket	bucket;
@@ -746,7 +757,7 @@ GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
 }
 #endif
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapResize(GSIMapTable map, uintptr_t new_capacity)
 {
   GSIMapBucket	new_buckets;
@@ -773,36 +784,25 @@ GSIMapResize(GSIMapTable map, uintptr_t new_capacity)
       size++;
     }
 
-#if     GS_WITH_GC
-  /* We don't need to use scanned memory because the nodes are not 'owned'
-   * by the bucket they are in, but rather are in chunks pointed to by
-   * the nodeChunks array.
-   */
-  new_buckets = (GSIMapBucket)NSAllocateCollectable
-    (size * sizeof(GSIMapBucket_t), 0);
-#else
   /* Use the zone specified for this map.
    */
   new_buckets = (GSIMapBucket)NSZoneCalloc(map->zone, size,
     sizeof(GSIMapBucket_t));
-#endif
 
   if (new_buckets != 0)
     {
       GSIMapRemangleBuckets(map, map->buckets, map->bucketCount, new_buckets,
 	size);
-#if     !GS_WITH_GC
       if (map->buckets != 0)
 	{
 	  NSZoneFree(map->zone, map->buckets);
 	}
-#endif
       map->buckets = new_buckets;
       map->bucketCount = size;
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRightSizeMap(GSIMapTable map, uintptr_t capacity)
 {
   /* FIXME: Now, this is a guess, based solely on my intuition.  If anyone
@@ -836,7 +836,7 @@ GSIMapRightSizeMap(GSIMapTable map, uintptr_t capacity)
  * is in progress.  The results of doing so are reasonably unpredictable.
  * <br />Remember, DON'T MESS WITH A MAP WHILE YOU'RE ENUMERATING IT.
  */
-static INLINE GSIMapEnumerator_t
+GS_STATIC_INLINE GSIMapEnumerator_t
 GSIMapEnumeratorForMap(GSIMapTable map)
 {
   GSIMapEnumerator_t	enumerator;
@@ -880,7 +880,7 @@ GSIMapEnumeratorForMap(GSIMapTable map)
 /**
  * Tidies up after map enumeration ... effectively destroys the enumerator.
  */
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapEndEnumerator(GSIMapEnumerator enumerator)
 {
   ((_GSIE)enumerator)->map = 0;
@@ -894,7 +894,7 @@ GSIMapEndEnumerator(GSIMapEnumerator enumerator)
  * bucket and node to remove the node from the map using the
  * GSIMapRemoveNodeFromMap() function.
  */
-static INLINE GSIMapBucket 
+GS_STATIC_INLINE GSIMapBucket 
 GSIMapEnumeratorBucket(GSIMapEnumerator enumerator)
 {
   if (((_GSIE)enumerator)->node != 0)
@@ -909,7 +909,7 @@ GSIMapEnumeratorBucket(GSIMapEnumerator enumerator)
 /**
  * Returns the next node in the map, or a nul pointer if at the end.
  */
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 {
   GSIMapNode	node = ((_GSIE)enumerator)->node;
@@ -917,7 +917,8 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 
   /* Find the frst available non-zeroed node.
    */
-  if (node != 0 && GSI_MAP_ZEROED(map) && GSI_MAP_READ_KEY(map, &node->key).addr == 0)
+  if (node != 0 && GSI_MAP_ZEROED(map)
+    && GSI_MAP_READ_KEY(map, &node->key).addr == 0)
     {
       uintptr_t		bucketCount = map->bucketCount;
       uintptr_t		bucket = ((_GSIE)enumerator)->bucket;
@@ -946,7 +947,7 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 	{
 	  uintptr_t	bucket = ((_GSIE)enumerator)->bucket;
 
-	  while (next != 0 && next->key.addr == 0)
+	  while (next != 0 && GSI_MAP_NODE_IS_EMPTY(map, next))
 	    {
 	      next = GSIMapRemoveAndFreeNode(map, bucket, next);
 	    }
@@ -962,7 +963,7 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 	      while (next == 0 && ++bucket < bucketCount)
 		{
 		  next = (map->buckets[bucket]).firstNode;
-		  while (next != 0 && next->key.addr == 0)
+		  while (next != 0 && GSI_MAP_NODE_IS_EMPTY(map, next))
 		    {
 		      next = GSIMapRemoveAndFreeNode(map, bucket, next);
 		    }
@@ -986,25 +987,24 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
  * Used to implement fast enumeration methods in classes that use GSIMap for
  * their data storage.
  */
-static INLINE NSUInteger 
+GS_STATIC_INLINE NSUInteger 
 GSIMapCountByEnumeratingWithStateObjectsCount(GSIMapTable map,
-                                              NSFastEnumerationState *state,
-                                              id *stackbuf,
-                                              NSUInteger len)
+  NSFastEnumerationState *state, id *stackbuf, NSUInteger len)
 {
   NSInteger count;
   NSInteger i;
 
-  /* We can store a GSIMapEnumerator inside the extra buffer in state on all
-   * platforms that don't suck beyond belief (i.e. everything except win64),
-   * but we can't on anything where long is 32 bits and pointers are 64 bits,
-   * so we have to construct it here to avoid breaking on that platform.
+  /* We can store a GSIMapEnumerator inside the extra buffer in state, but we
+   * need to handle platforms like Win64 where long is 32 bits and pointers are
+   * 64 bits, so we have to construct it here to avoid breaking on such
+   * platforms.
    */
   struct GSPartMapEnumerator
     {
       GSIMapNode node;
       uintptr_t bucket;
     };
+#define GS_PART_MAP_ENUMERATOR(state) ((struct GSPartMapEnumerator*)(uintptr_t)((state)->extra))
   GSIMapEnumerator_t enumerator;
 
   count = MIN(len, map->nodeCount - state->state);
@@ -1012,13 +1012,13 @@ GSIMapCountByEnumeratingWithStateObjectsCount(GSIMapTable map,
   /* Construct the real enumerator */
   if (0 == state->state)
     {
-        enumerator = GSIMapEnumeratorForMap(map);
+      enumerator = GSIMapEnumeratorForMap(map);
     }
   else
     {
       enumerator.map = map;
-      enumerator.node = ((struct GSPartMapEnumerator*)(state->extra))->node; 
-      enumerator.bucket = ((struct GSPartMapEnumerator*)(state->extra))->bucket;
+      enumerator.node = GS_PART_MAP_ENUMERATOR(state)->node; 
+      enumerator.bucket = GS_PART_MAP_ENUMERATOR(state)->bucket;
     }
   /* Get the next count objects and put them in the stack buffer. */
   for (i = 0; i < count; i++)
@@ -1034,8 +1034,8 @@ GSIMapCountByEnumeratingWithStateObjectsCount(GSIMapTable map,
         }
     }
   /* Store the important bits of the enumerator in the caller. */
-  ((struct GSPartMapEnumerator*)(state->extra))->node = enumerator.node;
-  ((struct GSPartMapEnumerator*)(state->extra))->bucket = enumerator.bucket;
+  GS_PART_MAP_ENUMERATOR(state)->node = enumerator.node;
+  GS_PART_MAP_ENUMERATOR(state)->bucket = enumerator.bucket;
   /* Update the rest of the state. */
   state->state += count;
   state->itemsPtr = stackbuf;
@@ -1043,7 +1043,7 @@ GSIMapCountByEnumeratingWithStateObjectsCount(GSIMapTable map,
 }
 
 #if	GSI_MAP_HAS_VALUE
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapAddPairNoRetain(GSIMapTable map, GSIMapKey key, GSIMapVal value)
 {
   GSIMapNode	node = map->freeNodes;
@@ -1052,10 +1052,6 @@ GSIMapAddPairNoRetain(GSIMapTable map, GSIMapKey key, GSIMapVal value)
     {
       GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
       node = map->freeNodes;
-      if (node == 0)
-	{
-	  return 0;
-	}
     }
   map->freeNodes = node->nextInBucket;
   GSI_MAP_WRITE_KEY(map, &node->key, key);
@@ -1066,7 +1062,7 @@ GSIMapAddPairNoRetain(GSIMapTable map, GSIMapKey key, GSIMapVal value)
   return node;
 }
 
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapAddPair(GSIMapTable map, GSIMapKey key, GSIMapVal value)
 {
   GSIMapNode	node = map->freeNodes;
@@ -1075,10 +1071,6 @@ GSIMapAddPair(GSIMapTable map, GSIMapKey key, GSIMapVal value)
     {
       GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
       node = map->freeNodes;
-      if (node == 0)
-	{
-	  return 0;
-	}
     }
   map->freeNodes = node->nextInBucket;
   GSI_MAP_WRITE_KEY(map, &node->key, key);
@@ -1091,7 +1083,7 @@ GSIMapAddPair(GSIMapTable map, GSIMapKey key, GSIMapVal value)
   return node;
 }
 #else
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapAddKeyNoRetain(GSIMapTable map, GSIMapKey key)
 {
   GSIMapNode	node = map->freeNodes;
@@ -1100,10 +1092,6 @@ GSIMapAddKeyNoRetain(GSIMapTable map, GSIMapKey key)
     {
       GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
       node = map->freeNodes;
-      if (node == 0)
-	{
-	  return 0;
-	}
     }
   map->freeNodes = node->nextInBucket;
   GSI_MAP_WRITE_KEY(map, &node->key, key);
@@ -1113,7 +1101,7 @@ GSIMapAddKeyNoRetain(GSIMapTable map, GSIMapKey key)
   return node;
 }
 
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapAddKey(GSIMapTable map, GSIMapKey key)
 {
   GSIMapNode	node = map->freeNodes;
@@ -1122,10 +1110,6 @@ GSIMapAddKey(GSIMapTable map, GSIMapKey key)
     {
       GSIMapMoreNodes(map, map->nodeCount < map->increment ? 0: map->increment);
       node = map->freeNodes;
-      if (node == 0)
-	{
-	  return 0;
-	}
     }
   map->freeNodes = node->nextInBucket;
   GSI_MAP_WRITE_KEY(map, &node->key, key);
@@ -1141,7 +1125,7 @@ GSIMapAddKey(GSIMapTable map, GSIMapKey key)
  * Removes the item for the specified key from the map.
  * If the key was present, returns YES, otherwise returns NO.
  */
-static INLINE BOOL
+GS_STATIC_INLINE BOOL
 GSIMapRemoveKey(GSIMapTable map, GSIMapKey key)
 {
   GSIMapBucket	bucket = GSIMapBucketForKey(map, key);
@@ -1157,7 +1141,7 @@ GSIMapRemoveKey(GSIMapTable map, GSIMapKey key)
   return NO;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapCleanMap(GSIMapTable map)
 {
   if (map->nodeCount > 0)
@@ -1196,13 +1180,15 @@ GSIMapCleanMap(GSIMapTable map)
 	  bucket->firstNode = 0;
 	  bucket++;
 	}
-      
-      prevNode->nextInBucket = map->freeNodes;
+      if (prevNode != 0)
+	{
+          prevNode->nextInBucket = map->freeNodes;
+	}
       map->freeNodes = startNode;
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapEmptyMap(GSIMapTable map)
 {
 #ifdef	GSI_MAP_NOCLEAN
@@ -1219,15 +1205,12 @@ GSIMapEmptyMap(GSIMapTable map)
 #endif
   if (map->buckets != 0)
     {
-#if	!GS_WITH_GC
       NSZoneFree(map->zone, map->buckets);
-#endif
       map->buckets = 0;
       map->bucketCount = 0;
     }
   if (map->nodeChunks != 0)
     {
-#if	!GS_WITH_GC
       unsigned int	i;
 
       for (i = 0; i < map->chunkCount; i++)
@@ -1235,7 +1218,6 @@ GSIMapEmptyMap(GSIMapTable map)
 	  NSZoneFree(map->zone, map->nodeChunks[i]);
 	}
       NSZoneFree(map->zone, map->nodeChunks);
-#endif
       map->chunkCount = 0;
       map->nodeChunks = 0;
     }
@@ -1243,7 +1225,7 @@ GSIMapEmptyMap(GSIMapTable map)
   map->zone = 0;
 }
 
-static INLINE void 
+GS_STATIC_INLINE void 
 GSIMapInitWithZoneAndCapacity(GSIMapTable map, NSZone *zone, uintptr_t capacity)
 {
   map->zone = zone;
@@ -1258,6 +1240,37 @@ GSIMapInitWithZoneAndCapacity(GSIMapTable map, NSZone *zone, uintptr_t capacity)
   GSIMapMoreNodes(map, capacity);
 }
 
+GS_STATIC_INLINE NSUInteger 
+GSIMapSize(GSIMapTable map)
+{
+  NSUInteger    index;
+  NSUInteger    size;
+  GSIMapNode	node;
+
+  /* Map table plus arrays of pointers to chunks
+   */
+  size = GSI_MAP_TABLE_S + map->chunkCount * sizeof(void*);
+
+  /* Add the array of buckets.
+   */
+  size += map->bucketCount * sizeof(GSIMapBucket_t);
+
+  /* Add the free nodes.
+   */
+  for (node = map->freeNodes; 0 != node; node = node->nextInBucket)
+    {
+      size += sizeof(GSIMapNode_t);
+    }
+
+  /* Add the used nodes (in the buckets).
+   */
+  for (index = 0; index < map->bucketCount; index++)
+    {
+      size += sizeof(GSIMapNode_t) * map->buckets[index].nodeCount;
+    }
+  return size;
+}
+
 #if	defined(__cplusplus)
 }
 #endif
