@@ -52,6 +52,9 @@ struct sockaddr_in;
   int			descriptor;
   BOOL			closeOnDealloc;
   BOOL			isStandardFile;
+  // stdin, stdout, and stderr
+  BOOL			isStandardStream;
+  BOOL			isStandardInput;
   BOOL			isNullDevice;
   BOOL			isSocket;
   BOOL			isNonBlocking;
