@@ -58,6 +58,7 @@
 
 @interface	NSURLRequest (Private)
 - (BOOL) _debug;
+- (id<GSLogDelegate>) _debugLogDelegate;
 - (id) _propertyForKey: (NSString*)key;
 - (void) _setProperty: (id)value forKey: (NSString*)key;
 @end
@@ -72,11 +73,23 @@
 
 
 @interface      NSURLProtocol (Private)
-+ (Class) _classToHandleRequest:(NSURLRequest *)request;
++ (Class) _classToHandleRequest: (NSURLRequest *)request;
++ (id<NSURLProtocolClient>) _ProtocolClient;
 @end
 
-/*
- * Internal class for handling HTTP authentication
+
+/* Internal class for handling HTTP protocol client messages
+ */
+@interface _NSURLProtocolClient : NSObject <NSURLProtocolClient>
+{
+  NSURLRequestCachePolicy  _cachePolicy;
+  NSMutableArray           *_cacheableData;
+  NSURLResponse            *_cacheableResponse;
+}
+@end
+
+
+/* Internal class for handling HTTP authentication
  */
 @class	NSLock;
 @interface GSHTTPAuthentication : NSObject
