@@ -61,7 +61,7 @@ typeSize(const char* type)
       case _C_ULNG_LNG:	return sizeof(unsigned long long);
       case _C_FLT:	return sizeof(float);
       case _C_DBL:	return sizeof(double);
-#if __GNUC__ > 2 && defined(_C_BOOL)
+#if defined(_C_BOOL) && (!defined(__GNUC__) || __GNUC__ > 2)
       case _C_BOOL:	return sizeof(_Bool);
 #endif
       case _C_PTR:	return sizeof(void*);
@@ -73,7 +73,7 @@ typeSize(const char* type)
 	{
 	  NSUInteger	size;
 
-	  NSGetSizeAndAlignment(type, &size, 0);
+	  NSGetSizeAndAlignment(type, &size, NULL);
 	  return (int)size;
 	}
       case _C_VOID:	return 0;
@@ -111,7 +111,7 @@ typeSize(const char* type)
         }
       size = strlen(type);
       objctype = (char *)NSZoneMalloc([self zone], size + 1);
-      strncpy(objctype, type, size);
+      memcpy(objctype, type, size);
       objctype[size] = '\0';
     }
   return self;
@@ -265,7 +265,7 @@ typeSize(const char* type)
   size = strlen(objctype)+1;
   [coder encodeValueOfObjCType: @encode(unsigned) at: &size];
   [coder encodeArrayOfObjCType: @encode(signed char) count: size at: objctype];
-  NSGetSizeAndAlignment(objctype, 0, &tsize);
+  NSGetSizeAndAlignment(objctype, &tsize, NULL);
   size = tsize;
   d = [NSMutableData new];
   [d serializeDataAt: data ofObjCType: objctype context: nil];
