@@ -54,8 +54,7 @@
 #define NAN 0x7fffffffffffffff
 #endif
 
-const NSTimeInterval NSTimeIntervalSince1970 = 978307200.0;
-NSString *const NSSystemClockDidChangeNotification = @"NSSystemClockDidChangeNotification";
+GS_DECLARE const NSTimeInterval NSTimeIntervalSince1970 = 978307200.0;
 
 
 
@@ -165,6 +164,12 @@ otherTime(NSDate* other)
     initWithTimeIntervalSinceReferenceDate: GSPrivateTimeNow()]);
 }
 
++ (instancetype) now
+{
+  return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
+    initWithTimeIntervalSinceReferenceDate: GSPrivateTimeNow()]);
+}
+
 /**
  * Returns an autoreleased instance representing the date and time given
  * by string. The value of string may be a 'natural' specification as
