@@ -46,13 +46,13 @@
 #import "Foundation/NSDateFormatter.h"
 #import "Foundation/NSCoder.h"
 
-#if defined(HAVE_UNICODE_UDAT_H)
+#if defined(HAVE_UNICODE_UDAT_H) && defined(HAVE_UNICODE_UDATPG_H)
 #define id id_ucal
 #include <unicode/udat.h>
 #undef id
-#endif
-#if defined(HAVE_UNICODE_UDATPG_H)
 #include <unicode/udatpg.h>
+#elif defined(HAVE_ICU_H)
+#include <icu.h>
 #endif
 
 
@@ -132,11 +132,11 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 {
   NSDateFormatter	*o = (id)NSCopyObject(self, 0, zone);
 
-  IF_NO_GC(RETAIN(o->_dateFormat));
+  IF_NO_ARC(RETAIN(o->_dateFormat);)
   if (0 != internal)
     {
       GS_COPY_INTERNAL(o, zone)
-      IF_NO_GC(RETAIN(GSIVar(o,_locale));)
+      IF_NO_ARC(RETAIN(GSIVar(o,_locale));)
 #if GS_USE_ICU == 1
       {
         UErrorCode err = U_ZERO_ERROR;
@@ -946,7 +946,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
       pat = malloc(sizeof(UChar) * patLength);
       [self->_dateFormat getCharacters: pat];
     }
-#if U_ICU_VERSION_MAJOR_NUM >= 50
+#if U_ICU_VERSION_MAJOR_NUM >= 50 || defined(HAVE_ICU_H)
   timeStyle = pat ? UDAT_PATTERN : NSToUDateFormatStyle (internal->_timeStyle);
   dateStyle = pat ? UDAT_PATTERN : NSToUDateFormatStyle (internal->_dateStyle);
 #else
@@ -1034,7 +1034,7 @@ symbolRange(NSInteger symbol, int *from)
       unichar           *value;
       NSString          *str;
       NSZone            *z = [self zone];
-      UErrorCode        err = U_ERROR_LIMIT;
+      UErrorCode        err = U_ZERO_ERROR;
       
       length
         = udat_getSymbols(internal->_formatter, symbol, idx, NULL, 0, &err);
