@@ -32,6 +32,7 @@
 #import "Foundation/NSHost.h"
 #import "Foundation/NSFileHandle.h"
 #import "Foundation/NSPathUtilities.h"
+#import "Foundation/NSURL.h"
 #import "GNUstepBase/GSTLS.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
 #import "GSPrivate.h"
@@ -41,23 +42,11 @@
 #define	EXPOSE_GSFileHandle_IVARS	1
 #import "GSFileHandle.h"
 
-// GNUstep Notification names
-
-NSString * const GSFileHandleConnectCompletionNotification
-  = @"GSFileHandleConnectCompletionNotification";
-NSString * const GSFileHandleWriteCompletionNotification
-  = @"GSFileHandleWriteCompletionNotification";
-
-// GNUstep key for getting error message.
-
-NSString * const GSFileHandleNotificationError
-  = @"GSFileHandleNotificationError";
-
 static Class NSFileHandle_abstract_class = nil;
 static Class NSFileHandle_concrete_class = nil;
 static Class NSFileHandle_ssl_class = nil;
 
-#if     defined(HAVE_GNUTLS) && !defined(_WIN32)
+#if     defined(HAVE_GNUTLS)
 @interface      GSTLSHandle : GSFileHandle
 {
 @public
@@ -93,7 +82,7 @@ static Class NSFileHandle_ssl_class = nil;
     {
       NSFileHandle_abstract_class = self;
       NSFileHandle_concrete_class = [GSFileHandle class];
-#if     defined(HAVE_GNUTLS) && !defined(_WIN32)
+#if     defined(HAVE_GNUTLS) 
       NSFileHandle_ssl_class = [GSTLSHandle class];
 #endif
     }
@@ -198,6 +187,36 @@ static Class NSFileHandle_ssl_class = nil;
   return AUTORELEASE([o initWithNullDevice]);
 }
 
++ (id) fileHandleForReadingFromURL: (NSURL*)url error:(NSError**)error
+{
+  id	o = [self fileHandleForReadingAtPath: [url path]];
+  if (!o && error)
+    {
+      *error = [NSError _last];
+    }
+  return o;
+}
+
++ (id) fileHandleForWritingToURL: (NSURL*)url error:(NSError**)error
+{
+  id	o = [self fileHandleForWritingAtPath: [url path]];
+  if (!o && error)
+    {
+      *error = [NSError _last];
+    }
+  return o;
+}
+
++ (id) fileHandleForUpdatingURL: (NSURL*)url error:(NSError**)error
+{
+  id	o = [self fileHandleForUpdatingAtPath: [url path]];
+  if (!o && error)
+    {
+      *error = [NSError _last];
+    }
+  return o;
+}
+
 /**
  *  Initialize with desc, which can point to either a regular file or
  *  socket connection.
@@ -452,35 +471,6 @@ static Class NSFileHandle_ssl_class = nil;
 
 @end
 
-// Keys for accessing userInfo dictionary in notification handlers.
-
-NSString * const NSFileHandleNotificationDataItem
-  = @"NSFileHandleNotificationDataItem";
-NSString * const NSFileHandleNotificationFileHandleItem
-  = @"NSFileHandleNotificationFileHandleItem";
-NSString * const NSFileHandleNotificationMonitorModes
-  = @"NSFileHandleNotificationMonitorModes";
-
-// Notification names
-
-NSString * const NSFileHandleConnectionAcceptedNotification
-  = @"NSFileHandleConnectionAcceptedNotification";
-NSString * const NSFileHandleDataAvailableNotification
-  = @"NSFileHandleDataAvailableNotification";
-NSString * const NSFileHandleReadCompletionNotification
-  = @"NSFileHandleReadCompletionNotification";
-NSString * const NSFileHandleReadToEndOfFileCompletionNotification
-  = @"NSFileHandleReadToEndOfFileCompletionNotification";
-
-// Exceptions
-
-/**
- * An exception used when a file error occurs.
- */
-NSString * const NSFileHandleOperationException
-  = @"NSFileHandleOperationException";
-
-
 // GNUstep class extensions
 
 /**
@@ -755,7 +745,7 @@ NSString * const NSFileHandleOperationException
     {
       NSRunLoop	*loop;
 
-      IF_NO_GC([self retain];)		// Don't get destroyed during runloop
+      IF_NO_ARC([self retain];)		// Don't get destroyed during runloop
       loop = [NSRunLoop currentRunLoop];
       [loop runUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.01]];
       if (NO == [self sslHandshakeEstablished: &result outgoing: NO])
@@ -799,7 +789,7 @@ NSString * const NSFileHandleOperationException
     {
       NSRunLoop	*loop;
 
-      IF_NO_GC([self retain];)		// Don't get destroyed during runloop
+      IF_NO_ARC([self retain];)		// Don't get destroyed during runloop
       loop = [NSRunLoop currentRunLoop];
       [loop runUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.01]];
       if (NO == [self sslHandshakeEstablished: &result outgoing: YES])
@@ -902,7 +892,7 @@ NSString * const NSFileHandleOperationException
 
 @end
 
-#if     defined(HAVE_GNUTLS) &&	!defined(_WIN32)
+#if     defined(HAVE_GNUTLS)
 
 /* Callback to allow the TLS code to pull data from the remote system.
  * If the operation fails, this sets the error number.
@@ -912,15 +902,33 @@ GSTLSHandlePull(gnutls_transport_ptr_t handle, void *buffer, size_t len)
 {
   ssize_t       result = 0;
   GSTLSHandle   *tls = (GSTLSHandle*)handle;
-  int           descriptor = [tls fileDescriptor];
+  int           descriptor = (int)(intptr_t)[tls nativeHandle];
 
-  result = read(descriptor, buffer, len);
+  result = recv(descriptor, buffer, len, 0);
   if (result < 0)
     {
 #if	HAVE_GNUTLS_TRANSPORT_SET_ERRNO
       if (tls->session && tls->session->session)
         {
-          gnutls_transport_set_errno (tls->session->session, errno);
+	  int	e;
+
+#if  defined(_WIN32)
+	  /* For windows, we need to map winsock errors to unix ones that
+	   * gnutls understands.
+	   */
+	  e = WSAGetLastError();
+	  if (WSAEWOULDBLOCK == e)
+	    {
+	      e = EAGAIN;
+	    }
+	  else if (WSAEINTR == e)
+	    {
+	      e = EINTR;
+	    }
+#else
+	  e = errno;
+#endif
+          gnutls_transport_set_errno (tls->session->session, e);
         }
 #endif
     }
@@ -935,15 +943,33 @@ GSTLSHandlePush(gnutls_transport_ptr_t handle, const void *buffer, size_t len)
 {
   ssize_t       result = 0;
   GSTLSHandle   *tls = (GSTLSHandle*)handle;
-  int           descriptor = [tls fileDescriptor];
+  int           descriptor = (int)(intptr_t)[tls nativeHandle];
 
-  result = write(descriptor, buffer, len);
+  result = send(descriptor, buffer, len, 0);
   if (result < 0)
     {
 #if	HAVE_GNUTLS_TRANSPORT_SET_ERRNO
       if (tls->session && tls->session->session)
         {
-          gnutls_transport_set_errno (tls->session->session, errno);
+	  int	e;
+
+#if  defined(_WIN32)
+	  /* For windows, we need to map winsock errors to unix ones that
+	   * gnutls understands.
+	   */
+	  e = WSAGetLastError();
+	  if (WSAEWOULDBLOCK == e)
+	    {
+	      e = EAGAIN;
+	    }
+	  else if (WSAEINTR == e)
+	    {
+	      e = EINTR;
+	    }
+#else
+	  e = errno;
+#endif
+          gnutls_transport_set_errno(tls->session->session, e);
         }
 #endif
     }
@@ -992,6 +1018,31 @@ GSTLSHandlePush(gnutls_transport_ptr_t handle, const void *buffer, size_t len)
   return [super read: buf length: len];
 }
 
+- (void) watchReadDescriptorForModes: (NSArray*)modes
+{
+  if (descriptor < 0)
+    { 
+      return;
+    }
+  if ([session pending] > 0)
+    {
+      NSRunLoop *l = [NSRunLoop currentRunLoop];
+
+      /* The underlying TLS buffers already have data so we signal
+       * an event as soon as possible.
+       */
+      [l performSelector: @selector(receivedEventRead)
+                  target: self
+                argument: nil
+                   order: 0
+                   modes: modes];
+    }
+  else
+    {
+      [super watchReadDescriptorForModes: modes];
+    }
+}
+
 - (BOOL) sslAccept
 {
   /* If a server session is over five minutes old, destroy it so that
@@ -1008,7 +1059,14 @@ GSTLSHandlePush(gnutls_transport_ptr_t handle, const void *buffer, size_t len)
 
 - (void) sslDisconnect
 {
-  [self setNonBlocking: NO];
+  /* When disconnecting, since the TCP/IP connection is not going to be
+   * re-used, we can use non-blocking I/O and abandon the cleanup in the
+   * TLS layer if the I/O cannot complete immediately.
+   * We don't want to block because a network issue (or a failure at the
+   * remote end) could tie up this thread until the 5 minute TCP/IP
+   * keepalive expires.
+   */
+  [self setNonBlocking: YES];
   [session disconnect: NO];
 }
 
@@ -1115,5 +1173,5 @@ GSTLSHandlePush(gnutls_transport_ptr_t handle, const void *buffer, size_t len)
 
 @end
 
-#endif  /* defined(HAVE_GNUTLS) && !defined(_WIN32) */
+#endif  /* defined(HAVE_GNUTLS) */
 
