@@ -64,7 +64,7 @@ static	NSIndexPath	*dummy = nil;
   return AUTORELEASE(o);
 }
 
-+ (NSIndexPath *) indexPathForItem: (NSInteger)item inSection: (NSInteger)section;
++ (NSIndexPath *) indexPathForItem: (NSInteger)item inSection: (NSInteger)section
 {
   NSUInteger idxs[2];
 
@@ -74,7 +74,7 @@ static	NSIndexPath	*dummy = nil;
   return [self indexPathWithIndexes: idxs length: 2];
 }
 
-+ (NSIndexPath *) indexPathForRow: (NSInteger)row inSection: (NSInteger)section;
++ (NSIndexPath *) indexPathForRow: (NSInteger)row inSection: (NSInteger)section
 {
   NSUInteger idxs[2];
 
