@@ -35,6 +35,7 @@
 
 #import "common.h"
 #include <objc/Protocol.h>
+#include <objc/message.h>
 #import "Foundation/NSMethodSignature.h"
 #import "Foundation/NSInvocation.h"
 #import "Foundation/NSLock.h"
@@ -131,7 +132,7 @@ GS_ROOT_CLASS @interface	NSZombie
 /* allocationLock is needed when for protecting the map table of zombie
  * information and if atomic operations are not available.
  */
-static pthread_mutex_t  allocationLock = PTHREAD_MUTEX_INITIALIZER;
+static gs_mutex_t  allocationLock = GS_MUTEX_INIT_STATIC;
 
 BOOL	NSZombieEnabled = NO;
 BOOL	NSDeallocateZombies = NO;
@@ -146,12 +147,12 @@ static void GSMakeZombie(NSObject *o, Class c)
   object_setClass(o, zombieClass);
   if (0 != zombieMap)
     {
-      pthread_mutex_lock(&allocationLock);
+      GS_MUTEX_LOCK(allocationLock);
       if (0 != zombieMap)
         {
           NSMapInsert(zombieMap, (void*)o, (void*)c);
         }
-      pthread_mutex_unlock(&allocationLock);
+      GS_MUTEX_UNLOCK(allocationLock);
     }
 }
 #endif
@@ -162,12 +163,12 @@ static void GSLogZombie(id o, SEL sel)
 
   if (0 != zombieMap)
     {
-      pthread_mutex_lock(&allocationLock);
+      GS_MUTEX_LOCK(allocationLock);
       if (0 != zombieMap)
         {
           c = NSMapGet(zombieMap, (void*)o);
         }
-      pthread_mutex_unlock(&allocationLock);
+      GS_MUTEX_UNLOCK(allocationLock);
     }
   if (c == 0)
     {
@@ -208,20 +209,6 @@ static void GSLogZombie(id o, SEL sel)
 #undef	GSATOMICREAD
 #endif
 
-
-/* Traditionally, GNUstep has been using a 32bit reference count in front
- * of the object. The automatic reference counting implementation in
- * libobjc2 uses an intptr_t instead, so NSObject will only be compatible
- * with ARC if either of the following apply:
- *
- * a) sizeof(intptr_t) == sizeof(int32_t)
- * b) we can provide atomic operations on pointer sized values, allowing
- *    us to extend the refcount to intptr_t.
- */
-#ifdef GS_ARC_COMPATIBLE
-#undef GS_ARC_COMPATIBLE
-#endif
-
 #if defined(__llvm__) || (defined(USE_ATOMIC_BUILTINS) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)))
 /* Use the GCC atomic operations with recent GCC versions */
 
@@ -230,7 +217,6 @@ typedef intptr_t gsrefcount_t;
 #define GSATOMICREAD(X) (*(X))
 #define GSAtomicIncrement(X)    __sync_add_and_fetch(X, 1)
 #define GSAtomicDecrement(X)    __sync_sub_and_fetch(X, 1)
-#define GS_ARC_COMPATIBLE 1
 
 #elif	defined(_WIN32)
 
@@ -395,6 +381,8 @@ GSAtomicDecrement(gsatomic_t X)
 
 #if	!defined(GSATOMICREAD)
 
+#include <pthread.h>
+
 typedef int     gsrefcount_t;   // No atomics, use a simple integer
 
 /* Having just one allocationLock for all leads to lock contention
@@ -422,16 +410,6 @@ static inline pthread_mutex_t   *GSAllocationLockForObject(id p)
 
 #endif
 
-#ifndef GS_ARC_COMPATIBLE
-/*
- * If we haven't previously declared that we can work in fast-ARC mode,
- * check whether a point is 32bit (4 bytes) wide, which also enables ARC
- * integration.
- */
-#  if __SIZEOF_POINTER__ == 4
-#    define GS_ARC_COMPATIBLE 1
-#  endif
-#endif
 
 #if defined(__GNUC__) && __GNUC__ < 4
 #define __builtin_offsetof(s, f) (uintptr_t)(&(((s*)0)->f))
@@ -841,12 +819,12 @@ NSDeallocateObject(id anObject)
 #ifdef OBJC_CAP_ARC
 	  if (0 != zombieMap)
 	    {
-              pthread_mutex_lock(&allocationLock);
+              GS_MUTEX_LOCK(allocationLock);
               if (0 != zombieMap)
                 {
                   NSMapInsert(zombieMap, (void*)anObject, (void*)aClass);
                 }
-              pthread_mutex_unlock(&allocationLock);
+              GS_MUTEX_UNLOCK(allocationLock);
 	    }
 	  if (NSDeallocateZombies == YES)
 	    {
@@ -946,8 +924,9 @@ NSShouldRetainWithZone (NSObject *anObject, NSZone *requestedZone)
  * </p>
  */
 @implementation NSObject
-#if  defined(GS_ARC_COMPATIBLE)
-- (void)_ARCCompliantRetainRelease {}
+#ifdef OBJC_CAP_ARC
++ (void) _TrivialAllocInit {}
+- (void) _ARCCompliantRetainRelease {}
 #endif
 
 /**
@@ -977,11 +956,14 @@ static id gs_weak_load(id obj)
   if (self == [NSObject class])
     {
 #ifdef _WIN32
-      {
-        // See libgnustep-base-entry.m
-        extern void gnustep_base_socket_init(void);
-        gnustep_base_socket_init();
-      }
+      /* Start of sockets so we can get host name and other info */
+      WORD wVersionRequested = MAKEWORD(2, 2);
+      WSADATA wsaData;
+      int wsaResult = WSAStartup(wVersionRequested, &wsaData);
+      if (wsaResult != 0)
+        {
+          fprintf(stderr, "Error %d initializing Windows Sockets\n", wsaResult);
+        }
 #else /* _WIN32 */
 
 #ifdef	SIGPIPE
@@ -1117,10 +1099,10 @@ static id gs_weak_load(id obj)
 + (void) _atExit
 {
   NSMapTable	*m = nil;
-  pthread_mutex_lock(&allocationLock);
+  GS_MUTEX_LOCK(allocationLock);
   m = zombieMap;
   zombieMap = nil;
-  pthread_mutex_unlock(&allocationLock);
+  GS_MUTEX_UNLOCK(allocationLock);
   DESTROY(m);
 }
 
@@ -1146,7 +1128,7 @@ static id gs_weak_load(id obj)
  *   instance is performed apart from setup to be an instance of
  *   the correct class: it is your responsibility to initialize the
  *   instance by calling an appropriate <code>init</code>
- *   method.  If you are not using the garbage collector, it is
+ *   method.  If you are not using ARC, it is
  *   also your responsibility to make sure the returned
  *   instance is destroyed when you finish using it, by calling
  *   the <code>release</code> method to destroy the instance
@@ -1199,7 +1181,7 @@ static id gs_weak_load(id obj)
  *   initialized instance, as would be generated by an
  *   <code>alloc</code> method followed by a corresponding
  *   <code>init...</code> method.  Please note that if you are
- *   not using a garbage collector, this means that instances
+ *   not using ARC, this means that instances
  *   generated by the <code>new...</code> methods are not
  *   autoreleased, that is, you are responsible for releasing
  *   (autoreleasing) the instances yourself.  So when you use
@@ -2099,12 +2081,6 @@ static id gs_weak_load(id obj)
 
   if (aSelector == 0)
     {
-      if (GSPrivateDefaultsFlag(GSMacOSXCompatible))
-	{
-	  [NSException raise: NSInvalidArgumentException
-		    format: @"%@ null selector given",
-	    NSStringFromSelector(_cmd)];
-	}
       return NO;
     }
 
@@ -2220,14 +2196,13 @@ static id gs_weak_load(id obj)
 /**
  * Sets the version number of the receiving class.  Should be nonnegative.
  */
-+ (id) setVersion: (NSInteger)aVersion
++ (void) setVersion: (NSInteger)aVersion
 {
   if (aVersion < 0)
     [NSException raise: NSInvalidArgumentException
 	        format: @"%s +setVersion: may not set a negative version",
 			GSClassNameFromObject(self)];
   class_setVersion(self, aVersion);
-  return self;
 }
 
 /**
@@ -2500,12 +2475,12 @@ static id gs_weak_load(id obj)
 
   if (0 != zombieMap)
     {
-      pthread_mutex_lock(&allocationLock);
+      GS_MUTEX_LOCK(allocationLock);
       if (0 != zombieMap)
         {
           c = NSMapGet(zombieMap, (void*)self);
         }
-      pthread_mutex_unlock(&allocationLock);
+      GS_MUTEX_UNLOCK(allocationLock);
     }
   return c;
 }
@@ -2527,9 +2502,9 @@ static id gs_weak_load(id obj)
     {
       return nil;
     }
-  pthread_mutex_lock(&allocationLock);
+  GS_MUTEX_LOCK(allocationLock);
   c = zombieMap ? NSMapGet(zombieMap, (void*)self) : Nil;
-  pthread_mutex_unlock(&allocationLock);
+  GS_MUTEX_UNLOCK(allocationLock);
 
   return [c instanceMethodSignatureForSelector: aSelector];
 }
