@@ -164,7 +164,7 @@ For more detailed assistance, please report the error to bug-gnustep@gnu.org.\n\
  *************************************************************************/
 @interface _NSConcreteProcessInfo: NSProcessInfo
 - (id) autorelease;
-- (void) release;
+- (oneway void) release;
 - (id) retain;
 @end
 
@@ -174,7 +174,7 @@ For more detailed assistance, please report the error to bug-gnustep@gnu.org.\n\
   return self;
 }
 
-- (void) release
+- (oneway void) release
 {
   return;
 }
@@ -306,7 +306,7 @@ _gnu_process_args(int argc, char *argv[], char *env[])
     }
 
   /* Getting the process name */
-  IF_NO_GC(RELEASE(_gnu_processName));
+  IF_NO_ARC(RELEASE(_gnu_processName);)
   _gnu_processName = [arg0 lastPathComponent];
 #if	defined(_WIN32)
   /* On windows we remove any .exe extension for consistency with app names
@@ -321,7 +321,7 @@ _gnu_process_args(int argc, char *argv[], char *env[])
       }
   }
 #endif
-  IF_NO_GC(RETAIN(_gnu_processName));
+  IF_NO_ARC(RETAIN(_gnu_processName);)
 
   /* Copy the argument list */
 #if	defined(_WIN32)
@@ -349,7 +349,7 @@ _gnu_process_args(int argc, char *argv[], char *env[])
 	}
     }
     
-  IF_NO_GC(RELEASE(_gnu_arguments));
+  IF_NO_ARC(RELEASE(_gnu_arguments);)
   _gnu_arguments = [[NSArray alloc] initWithObjects: obj_argv count: added];
   RELEASE(arg0);
 }
@@ -376,7 +376,7 @@ _gnu_process_args(int argc, char *argv[], char *env[])
 	    obj_argv[added++] = str;
 	}
 
-      IF_NO_GC(RELEASE(_gnu_arguments));
+      IF_NO_ARC(RELEASE(_gnu_arguments);)
       _gnu_arguments = [[NSArray alloc] initWithObjects: obj_argv count: added];
       RELEASE(arg0);
     }
@@ -458,11 +458,11 @@ _gnu_process_args(int argc, char *argv[], char *env[])
 	    i++;
 	  }
       }
-    IF_NO_GC(RELEASE(_gnu_environment));
+    IF_NO_ARC(RELEASE(_gnu_environment);)
     _gnu_environment = [[NSDictionary alloc] initWithObjects: values
 						     forKeys: keys];
-    IF_NO_GC(RELEASE(keys));
-    IF_NO_GC(RELEASE(values));
+    IF_NO_ARC(RELEASE(keys);)
+    IF_NO_ARC(RELEASE(values);)
   }
   [arp drain];
 }
@@ -987,21 +987,6 @@ int main(int argc, char *argv[], char *env[])
          sizeof(_NSConstantStringClassReference));
 #endif
 
-#if defined(_WIN32)
-  WSADATA lpWSAData;
-
-  // Initialize Windows Sockets
-  if (WSAStartup(MAKEWORD(1,1), &lpWSAData))
-    {
-      printf("Could not startup Windows Sockets\n");
-      exit(1);
-    }
-#endif /* _WIN32 */
-
-#ifdef __MS_WIN__
-  _MB_init_runtime();
-#endif /* __MS_WIN__ */
-
   _gnu_process_args(argc, argv, env);
 
   /* Call the user defined main function */
@@ -1139,7 +1124,7 @@ static void determineOperatingSystem()
        * use the information from NSBundle and only get the version info
        * here.
        */
-      _operatingSystemVersion = [[NSString alloc] initWithFormat: @"%d.%d",
+      _operatingSystemVersion = [[NSString alloc] initWithFormat: @"%lu.%lu",
         osver.dwMajorVersion, osver.dwMinorVersion];
 #else
 #if	defined(HAVE_SYS_UTSNAME_H)
@@ -1568,22 +1553,28 @@ GSInitializeProcess(int argc, char **argv, char **envp)
 }
 
 #ifdef __ANDROID__
+static NSString *
+_NSStringFromJString(JNIEnv *env, jstring jstr)
+{
+  const jchar *unichars = (*env)->GetStringChars(env, jstr, NULL);
+  jsize len = (*env)->GetStringLength(env, jstr);
+  NSString *result = [NSString stringWithCharacters:unichars length:len];
+  (*env)->ReleaseStringChars(env, jstr, unichars);
+  return result;
+}
+
 void
 GSInitializeProcessAndroid(JNIEnv *env, jobject context)
 {
-  [NSProcessInfo class];
-
-  // create global reference to to prevent garbage collection
-  _androidContext = (*env)->NewGlobalRef(env, context);
+  jclass contextCls = (*env)->GetObjectClass(env, context);
 
   // get package code path (path to APK)
-  jclass cls = (*env)->GetObjectClass(env, context);
-  jmethodID packageCodePathMethod = (*env)->GetMethodID(env, cls, "getPackageCodePath", "()Ljava/lang/String;");
+  jmethodID packageCodePathMethod = (*env)->GetMethodID(env, contextCls, "getPackageCodePath", "()Ljava/lang/String;");
   jstring packageCodePathJava = (*env)->CallObjectMethod(env, context, packageCodePathMethod);
   const char *packageCodePath = (*env)->GetStringUTFChars(env, packageCodePathJava, NULL);
 
   // get package name
-  jmethodID packageNameMethod = (*env)->GetMethodID(env, cls, "getPackageName", "()Ljava/lang/String;");
+  jmethodID packageNameMethod = (*env)->GetMethodID(env, contextCls, "getPackageName", "()Ljava/lang/String;");
   jstring packageNameJava = (*env)->CallObjectMethod(env, context, packageNameMethod);
   const char *packageName = (*env)->GetStringUTFChars(env, packageNameJava, NULL);
 
@@ -1597,45 +1588,82 @@ GSInitializeProcessAndroid(JNIEnv *env, jobject context)
   char *arg0;
   asprintf(&arg0, "%.*s/%s", (int)(lastSlash - packageCodePath), packageCodePath, packageName);
 
+
+  // get current locale
+  jclass localeCls = (*env)->FindClass(env, "java/util/Locale");
+  jmethodID localeDefaultMethod = (*env)->GetStaticMethodID(env, localeCls, "getDefault", "()Ljava/util/Locale;");
+  jmethodID localeIdMethod = (*env)->GetMethodID(env, localeCls, "toLanguageTag", "()Ljava/lang/String;");
+  jobject localeObj = (*env)->CallStaticObjectMethod(env, localeCls, localeDefaultMethod);
+  jstring localeIdJava = (*env)->CallObjectMethod(env, localeObj, localeIdMethod);
+  const char *localeIdOrig = (*env)->GetStringUTFChars(env, localeIdJava, NULL);
+
+  // Android uses dashes as delimiters (e.g "en-US"), but we expect underscores
+  char *localeId = strdup(localeIdOrig);
+  for (int i = 0; localeId[i]; i++) {
+    if (localeId[i] == '-') {
+      localeId[i] = '_';
+    }
+  }
+
+  // get current time zone
+  jclass timezoneCls = (*env)->FindClass(env, "java/util/TimeZone");
+  jmethodID timezoneDefaultMethod = (*env)->GetStaticMethodID(env, timezoneCls, "getDefault", "()Ljava/util/TimeZone;");
+  jmethodID timezoneIdMethod = (*env)->GetMethodID(env, timezoneCls, "getID", "()Ljava/lang/String;");
+  jobject timezoneObj = (*env)->CallStaticObjectMethod(env, timezoneCls, timezoneDefaultMethod);
+  jstring timezoneIdJava = (*env)->CallObjectMethod(env, timezoneObj, timezoneIdMethod);
+  const char *timezoneId = (*env)->GetStringUTFChars(env, timezoneIdJava, NULL);
+
+  // initialize process with these options
+  char *argv[] = {
+    arg0,
+    "-Locale", localeId,
+    "-Local Time Zone", (char *)timezoneId,
+    "-GSLogSyslog", "YES" // use syslog (available via logcat) instead of stdout/stderr (not available on Android)
+  };
+
+  GSInitializeProcessAndroidWithArgs(env, context, sizeof(argv)/sizeof(char *), argv, NULL);
+
+  free(arg0);
   (*env)->ReleaseStringUTFChars(env, packageCodePathJava, packageCodePath);
   (*env)->ReleaseStringUTFChars(env, packageNameJava, packageName);
+  (*env)->ReleaseStringUTFChars(env, localeIdJava, localeId);
+  (*env)->ReleaseStringUTFChars(env, timezoneIdJava, timezoneId);
+}
+
+void
+GSInitializeProcessAndroidWithArgs(JNIEnv *env, jobject context, int argc, char **argv, char **envp)
+{
+  [NSProcessInfo class];
+
+  // create global reference to to prevent garbage collection
+  _androidContext = (*env)->NewGlobalRef(env, context);
 
   // initialize process
   [procLock lock];
   fallbackInitialisation = YES;
-  char *argv[] = {
-    arg0,
-    "-GSLogSyslog", "YES" // use syslog (available via logcat) instead of stdout/stderr (not available on Android)
-  };
-  _gnu_process_args(sizeof(argv)/sizeof(char *), argv, NULL);
+  _gnu_process_args(argc, argv, NULL);
   [procLock unlock];
 
-  free(arg0);
+  jclass contextCls = (*env)->GetObjectClass(env, context);
 
   // get File class and path method
   jclass fileCls = (*env)->FindClass(env, "java/io/File");
   jmethodID getAbsolutePathMethod = (*env)->GetMethodID(env, fileCls, "getAbsolutePath", "()Ljava/lang/String;");
 
   // get Android files dir
-  jmethodID filesDirMethod = (*env)->GetMethodID(env, cls, "getFilesDir", "()Ljava/io/File;");
+  jmethodID filesDirMethod = (*env)->GetMethodID(env, contextCls, "getFilesDir", "()Ljava/io/File;");
   jobject filesDirObj = (*env)->CallObjectMethod(env, context, filesDirMethod);
   jstring filesDirJava = (*env)->CallObjectMethod(env, filesDirObj, getAbsolutePathMethod);
-	const jchar *filesDirUnichars = (*env)->GetStringChars(env, filesDirJava, NULL);
-  jsize filesDirLength = (*env)->GetStringLength(env, filesDirJava);
-  _androidFilesDir = [NSString stringWithCharacters:filesDirUnichars length:filesDirLength];
-  (*env)->ReleaseStringChars(env, filesDirJava, filesDirUnichars);
+  _androidFilesDir = _NSStringFromJString(env, filesDirJava);
 
   // get Android cache dir
-  jmethodID cacheDirMethod = (*env)->GetMethodID(env, cls, "getCacheDir", "()Ljava/io/File;");
+  jmethodID cacheDirMethod = (*env)->GetMethodID(env, contextCls, "getCacheDir", "()Ljava/io/File;");
   jobject cacheDirObj = (*env)->CallObjectMethod(env, context, cacheDirMethod);
   jstring cacheDirJava = (*env)->CallObjectMethod(env, cacheDirObj, getAbsolutePathMethod);
-	const jchar *cacheDirUnichars = (*env)->GetStringChars(env, cacheDirJava, NULL);
-  jsize cacheDirLength = (*env)->GetStringLength(env, cacheDirJava);
-  _androidCacheDir = [NSString stringWithCharacters:cacheDirUnichars length:cacheDirLength];
-  (*env)->ReleaseStringChars(env, cacheDirJava, cacheDirUnichars);
+  _androidCacheDir = _NSStringFromJString(env, cacheDirJava);
 
   // get asset manager and initialize NSBundle
-  jmethodID assetManagerMethod = (*env)->GetMethodID(env, cls, "getAssets", "()Landroid/content/res/AssetManager;");
+  jmethodID assetManagerMethod = (*env)->GetMethodID(env, contextCls, "getAssets", "()Landroid/content/res/AssetManager;");
   jstring assetManagerJava = (*env)->CallObjectMethod(env, context, assetManagerMethod);
   [NSBundle setJavaAssetManager:assetManagerJava withJNIEnv:env];
 
