@@ -22,16 +22,17 @@
    Boston, MA 02110 USA.
 
    <title>NSTask class reference</title>
-   $Date$ $Revision$
    */
 
 #import "common.h"
 #define	EXPOSE_NSTask_IVARS	1
+#import "Foundation/FoundationErrors.h"
 #import "Foundation/NSAutoreleasePool.h"
 #import "Foundation/NSCharacterSet.h"
 #import "Foundation/NSData.h"
 #import "Foundation/NSDate.h"
 #import "Foundation/NSEnumerator.h"
+#import "Foundation/NSError.h"
 #import "Foundation/NSException.h"
 #import "Foundation/NSFileHandle.h"
 #import "Foundation/NSFileManager.h"
@@ -339,7 +340,7 @@ pty_slave(const char* name)
   if (_currentDirectoryPath == nil)
     {
       [self setCurrentDirectoryPath:
-		[[NSFileManager defaultManager] currentDirectoryPath]];
+	[[NSFileManager defaultManager] currentDirectoryPath]];
     }
   return _currentDirectoryPath;
 }
@@ -420,7 +421,7 @@ pty_slave(const char* name)
   if (NO == [self launchAndReturnError: &e])
     {
       [NSException raise: NSInvalidArgumentException
-		  format: @"%@", e ? e : @"Unable to launch"];
+		  format: @"%@", e ? [e description] : @"Unable to launch"];
     }
 }
 
@@ -903,7 +904,45 @@ pty_slave(const char* name)
 
 - (BOOL) launchAndReturnError: (NSError **)error
 {
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  NSString	*cwd;
+  BOOL		ok;
+
   ASSIGN(_launchingThread, [NSThread currentThread]);
+
+  cwd = [self currentDirectoryPath];
+  if (NO == [mgr fileExistsAtPath: cwd isDirectory: &ok])
+    {
+      if (error)
+	{
+	  NSDictionary	*info = [NSDictionary dictionaryWithObjectsAndKeys:
+	    @"does not exist", NSLocalizedDescriptionKey,
+	    cwd, NSFilePathErrorKey,
+	    nil];
+	  *error = [NSError errorWithDomain: NSCocoaErrorDomain
+				       code: NSFileNoSuchFileError
+				   userInfo: info];
+	}
+      return NO;
+    }
+  if (NO == ok)
+    {
+      if (error)
+	{
+	  *error = [NSError _systemError: ENOTDIR];
+	  [*error _setObject: cwd forKey: NSFilePathErrorKey];
+	}
+      return NO;
+    }
+  if (NO == [mgr isExecutableFileAtPath: cwd])
+    {
+      if (error)
+	{
+	  *error = [NSError _systemError: EACCES];
+	  [*error _setObject: cwd forKey: NSFilePathErrorKey];
+	}
+      return NO;
+    }
   return YES;
 }
 
@@ -969,7 +1008,7 @@ pty_slave(const char* name)
             coalesceMask: NSNotificationNoCoalescing
                 forModes: nil];
 
-  if (_handler != nil)
+  if (_handler != NULL)
     {
       CALL_BLOCK_NO_ARGS(_handler);
     }
@@ -1215,7 +1254,10 @@ quotedFromString(NSString *aString)
       return NO;
     }
 
-  [super launchAndReturnError: error];
+  if (NO == [super launchAndReturnError: error])
+    {
+      return NO;
+    }
 
   lpath = [self _fullLaunchPath];
   wexecutable = (const unichar*)[lpath fileSystemRepresentation];
@@ -1526,6 +1568,7 @@ GSPrivateCheckTasks()
 // 10.13 method...
 - (BOOL) launchAndReturnError: (NSError **)error
 {
+  NSDictionary      	*info;
   NSMutableArray	*toClose;
   NSString      	*lpath;
   int			pid;
@@ -1548,8 +1591,6 @@ GSPrivateCheckTasks()
     {
       if (error)
 	{
-	  NSDictionary      *info;
-
 	  info = [NSDictionary dictionaryWithObjectsAndKeys:
 	    @"task has already been launched", NSLocalizedDescriptionKey, nil];
 	  *error = [NSError errorWithDomain: NSCocoaErrorDomain
@@ -1559,7 +1600,10 @@ GSPrivateCheckTasks()
       return NO;
     }
 
-  [super launchAndReturnError: error];
+  if (NO == [super launchAndReturnError: error])
+    {
+      return NO;
+    }
 
   lpath = [self _fullLaunchPath];
   executable = [lpath fileSystemRepresentation];
@@ -1728,14 +1772,30 @@ GSPrivateCheckTasks()
       /*
        * Close any extra descriptors.
        */
+#if	defined(HAVE_CLOSEFROM)
+      closefrom(3);
+#elif	defined(F_CLOSEM)
+      (void)fcntl(3, F_CLOSEM, 0);
+#elif	defined(_SC_OPEN_MAX)
+      i = sysconf(_SC_OPEN_MAX);
+      while (i-- > 3)
+	{
+	  (void)close(i);
+	}
+#else
       for (i = 3; i < NOFILE; i++)
 	{
-	  (void) close(i);
+	  (void)close(i);
 	}
+#endif
+
+      if (0 != chdir(path))
+        {
+          _exit(-1);
+        }
 
-      (void)chdir(path);
       (void)execve(executable, (char**)args, (char**)envl);
-      exit(-1);
+      _exit(-1);
     }
   else
     {
