@@ -106,6 +106,7 @@
 #import "Foundation/NSTimeZone.h"
 #import "Foundation/NSByteOrder.h"
 #import "Foundation/NSLocale.h"
+#import "Foundation/NSScanner.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
 #import "GSPrivate.h"
 #import "GSPThread.h"
@@ -153,6 +154,11 @@
 /* Many systems have this file */
 #define SYSTEM_TIME_FILE @"/etc/localtime"
 
+/* Include public domain code (modified for use here) to parse standard
+ * posix time zone files.
+ */
+#include "tzdb.h"
+
 /* If TZDIR told us where the zoneinfo files are, don't append anything else */
 #ifdef TZDIR
 #define POSIX_TZONES     @""
@@ -163,11 +169,6 @@
 #define BUFFER_SIZE 512
 #define WEEK_MILLISECONDS (7.0*24.0*60.0*60.0*1000.0)
 
-/* Include public domain code (modified for use here) to parse standard
- * posix time zone files.
- */
-#include "tzdb.h"
-
 #if GS_USE_ICU == 1
 static inline int
 _NSToICUTZDisplayStyle(NSTimeZoneNameStyle style)
@@ -806,6 +807,7 @@ static NSMapTable	*absolutes = 0;
 
 - (void) dealloc
 {
+  RELEASE(abbrev);
   RELEASE(timeZone);
   DEALLOC
 }
@@ -824,7 +826,7 @@ static NSMapTable	*absolutes = 0;
 		withDST: (BOOL)isDST
 {
   timeZone = RETAIN(aZone);
-  abbrev = anAbbrev;		// NB. Depend on this being retained in aZone
+  abbrev = RETAIN(anAbbrev);
   offset = anOffset;
   is_dst = isDST;
   return self;
@@ -1777,7 +1779,7 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
  * Each element contains an array of NSStrings which are
  * the region names.
  */
-+ (NSArray*) timeZoneArray
++ (NSArray *)timeZoneArray
 {
   static NSArray *regionsArray = nil;
 
@@ -1789,53 +1791,60 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
   GS_MUTEX_LOCK(zone_mutex);
   if (regionsArray == nil)
     {
-      NSAutoreleasePool	*pool = [NSAutoreleasePool new];
-      int		index;
-      int		i;
-      char		name[80];
-      FILE		*fp;
+      NSAutoreleasePool *pool = [NSAutoreleasePool new];
       NSMutableArray	*temp_array[24];
+      NSInteger		 index;
+      NSInteger		 i;
+      NSString		*name;
       NSString		*path;
+      NSString		*contents;
+      NSScanner		*scanner;
+      NSCharacterSet	*newLineSet;
+      NSError		*error = nil;
 
       for (i = 0; i < 24; i++)
 	{
 	  temp_array[i] = [NSMutableArray array];
 	}
 
-      path = _time_zone_path (REGIONS_FILE, nil);
+      path = _time_zone_path(REGIONS_FILE, nil);
       if (path != nil)
 	{
-#if	defined(_WIN32)
-	  unichar	mode[3];
-
-	  mode[0] = 'r';
-	  mode[1] = 'b';
-	  mode[2] = '\0';
-
-	  fp = _wfopen((const unichar*)[path fileSystemRepresentation], mode);
-#else
-	  fp = fopen([path fileSystemRepresentation], "r");
-#endif
-	  if (fp == NULL)
+	  contents = [NSString stringWithContentsOfFile: path
+					       encoding: NSUTF8StringEncoding
+						  error: &error];
+	  if (!contents)
 	    {
-              GS_MUTEX_UNLOCK(zone_mutex);
-	      [NSException
-		raise: NSInternalInconsistencyException
-		format: @"Failed to open time zone regions array file."];
+	      NSException  *exp;
+	      NSDictionary *userInfo;
+	      GS_MUTEX_UNLOCK(zone_mutex);
+
+	      userInfo = [NSDictionary
+		dictionaryWithObjectsAndKeys: error, @"underlyingError"];
+	      exp = [NSException
+		exceptionWithName: NSInternalInconsistencyException
+			   reason:
+			     @"Failed to open time zone regions array file."
+			 userInfo: userInfo];
+	      [exp raise];
 	    }
-	  while (fscanf(fp, "%d %79s", &index, name) == 2)
+	  newLineSet = [NSCharacterSet newlineCharacterSet];
+	  scanner = [NSScanner scannerWithString: contents];
+
+	  while ([scanner scanInteger: &index] &&
+		 [scanner scanUpToCharactersFromSet: newLineSet
+					 intoString: &name])
 	    {
-              if (index < 0)
-                index = 0;
-              else
-                index %= 24;
-	      [temp_array[index]
-		addObject: [NSString stringWithUTF8String: name]];
+	      if (index < 0)
+		index = 0;
+	      else
+		index %= 24;
+
+	      [temp_array[index] addObject: name];
 	    }
-	  fclose(fp);
 	}
       else
-	{
+	  {
 	  NSString	*zonedir = [NSTimeZone _getTimeZoneFile: @"WET"]; 
 
 	  if (tzdir != nil)
@@ -1849,12 +1858,17 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
 	      while ((name = [enumerator nextObject]) != nil)
 		{
 		  NSTimeZone	*zone = nil;
+		  NSString	*ext;
 		  BOOL		isDir;
 		
 		  path = [zonedir stringByAppendingPathComponent: name];
+		  ext = [path pathExtension];
 		  if ([mgr fileExistsAtPath: path isDirectory: &isDir]
                     && isDir == NO
-                    && [[path pathExtension] isEqual: @"tab"] == NO)
+                    && [ext isEqual: @"tab"] == NO
+                    && [ext isEqual: @"zi"] == NO
+                    && [ext isEqual: @"list"] == NO
+                    && [ext isEqual: @"leapseconds"] == NO)
 		    {
 		      zone = [zoneDictionary objectForKey: name];
 		      if (zone == nil)
@@ -2965,7 +2979,7 @@ newDetailInZoneForType(GSTimeZone *zone, TypeInfo *type)
 static TypeInfo
 getTypeInfo(NSTimeInterval since, GSTimeZone *zone)
 {
-  int64_t       when = (int64_t)since;
+  time_t        when = (time_t)since;
   gstm		tm;
   TypeInfo      type;
 
