@@ -47,6 +47,11 @@
 #import "GNUstepBase/NSString+GNUstepBase.h"
 #import "GNUstepBase/NSURL+GNUstepBase.h"
 
+@interface GSMimeHeader (HTTPRequest)
+- (void) addToBuffer: (NSMutableData*)buf
+             masking: (NSMutableData**)masked;
+@end
+
 /* Define to 1 for experimental (net yet working) compression support
  */
 #ifdef	USE_ZLIB
@@ -355,8 +360,9 @@ static NSLock		*pairLock = nil;
   float			_version;	// The HTTP version in use.
   int			_statusCode;	// The HTTP status code returned.
   NSInputStream		*_body;		// for sending the body
-  unsigned		_writeOffset;	// Request data to write
-  NSData		*_writeData;	// Request bytes written so far
+  unsigned		_writeOffset;	// Request bytes written so far
+  NSData		*_writeData;	// Request data to write
+  NSData		*_masked;	// Request masked data
   BOOL			_complete;
   BOOL			_debug;
   BOOL			_isLoading;
@@ -715,6 +721,11 @@ typedef struct {
   return NO;
 }
 
++ (BOOL) canInitWithTask: (NSURLSessionTask*)task
+{
+  return NO;
+}
+
 + (NSURLRequest *) canonicalRequestForRequest: (NSURLRequest *)request
 {
   return request;
@@ -780,6 +791,8 @@ typedef struct {
   [_body release];			// for sending the body
   [_response release];
   [_credential release];
+  DESTROY(_writeData);
+  DESTROY(_masked);
   [super dealloc];
 }
 
@@ -949,6 +962,8 @@ typedef struct {
                 GSTLSCertificateKeyFile,
                 GSTLSCertificateKeyPassword,
                 GSTLSDebug,
+                GSTLSIssuers,
+                GSTLSOwners,
                 GSTLSPriority,
                 GSTLSRemoteHosts,
                 GSTLSRevokeFile,
@@ -1002,6 +1017,7 @@ typedef struct {
     }
   _isLoading = NO;
   DESTROY(_writeData);
+  DESTROY(_masked);
   if (this->input != nil)
     {
       [this->input setDelegate: nil];
@@ -1533,17 +1549,24 @@ typedef struct {
 	    {
 	      if (_debug)
 		{
-		  if (NO == [_logDelegate putBytes: bytes + _writeOffset
+		  const unsigned char   *b = [_masked bytes];
+
+		  if (NULL == b)
+		    {
+		      b = bytes;
+		    }
+		  if (NO == [_logDelegate putBytes: b + _writeOffset
 					  ofLength: written
 					  byHandle: self])
 		    {
-		      debugWrite(self, written, bytes + _writeOffset);
+		      debugWrite(self, written, b + _writeOffset);
 		    }
 		}
 	      _writeOffset += written;
 	      if (_writeOffset >= len)
 		{
 		  DESTROY(_writeData);
+		  DESTROY(_masked);
 		  if (_body == nil)
 		    {
 		      _body = RETAIN([this->request HTTPBodyStream]);
@@ -1676,7 +1699,7 @@ typedef struct {
 {
   /* Make sure no action triggered by anything else destroys us prematurely.
    */
-  IF_NO_GC([[self retain] autorelease];)
+  IF_NO_ARC([[self retain] autorelease];)
 
 #if 0
   NSLog(@"stream: %@ handleEvent: %x for: %@ (ip %p, op %p)",
@@ -1710,11 +1733,13 @@ typedef struct {
 	  case NSStreamEventOpenCompleted: 
 	    {
 	      NSMutableData	*m;
+	      NSMutableData	*mm = nil;
 	      NSDictionary	*d;
 	      NSEnumerator	*e;
 	      NSString		*s;
 	      NSURL		*u;
 	      int		l;		
+	      NSData		*bytes;
 
 	      if (_debug)
 	        {
@@ -1733,6 +1758,7 @@ typedef struct {
                 [stream propertyForKey: GSStreamRemoteAddressKey],
                 [stream propertyForKey: GSStreamRemotePortKey]];
 	      DESTROY(_writeData);
+	      DESTROY(_masked);
 	      _writeOffset = 0;
 	      if ([this->request HTTPBodyStream] == nil)
 	        {
@@ -1783,8 +1809,14 @@ typedef struct {
                   h = [[GSMimeHeader alloc] initWithName: s
                                                    value: [d objectForKey: s]
                                               parameters: nil];
-                  [m appendData:
-                    [h rawMimeDataPreservingCase: YES foldedAt: 0]];
+		  if (_debug || mm)
+		    {
+		      [h addToBuffer: m masking: &mm];
+		    }
+		  else
+		    {
+		      [h addToBuffer: m masking: NULL];
+		    }
                   RELEASE(h);
 		}
 
@@ -1801,6 +1833,10 @@ typedef struct {
                   static char   *ct
                     = "Content-Type: application/x-www-form-urlencoded\r\n";
 		  [m appendBytes: ct length: strlen(ct)];
+		  if (mm)
+		    {
+		      [mm appendBytes: ct length: strlen(ct)];
+		    }
 		}
 	      if ([this->request valueForHTTPHeaderField: @"Host"] == nil)
 		{
@@ -1830,16 +1866,31 @@ typedef struct {
 		    {
 		      s = [NSString stringWithFormat: @"Host: %@:%@\r\n", h, p];
 		    }
-                  [m appendData: [s dataUsingEncoding: NSASCIIStringEncoding]];
+                  bytes = [s dataUsingEncoding: NSASCIIStringEncoding];
+                  [m appendData: bytes];
+		  if (mm)
+		    {
+		      [mm appendData: bytes];
+		    }
 		}
 	      if (l >= 0 && [this->request
 	        valueForHTTPHeaderField: @"Content-Length"] == nil)
 		{
                   s = [NSString stringWithFormat: @"Content-Length: %d\r\n", l];
-                  [m appendData: [s dataUsingEncoding: NSASCIIStringEncoding]];
+                  bytes = [s dataUsingEncoding: NSASCIIStringEncoding];
+                  [m appendData: bytes];
+		  if (mm)
+		    {
+		      [mm appendData: bytes];
+		    }
 		}
 	      [m appendBytes: "\r\n" length: 2];	// End of headers
-	      _writeData  = m;
+	      if (mm)
+		{
+		  [mm appendBytes: "\r\n" length: 2];
+		}
+	      _writeData = m;
+	      ASSIGN(_masked, mm);
 	    }			// Fall through to do the write
 
 	  case NSStreamEventHasSpaceAvailable: 
