@@ -39,6 +39,7 @@ typedef struct {
   NSMutableDictionary		*headers;
   BOOL				shouldHandleCookies;
   BOOL                          debug;
+  id<GSLogDelegate>             ioDelegate;
   NSURL				*URL;
   NSURL				*mainDocumentURL;
   NSURLRequestCachePolicy	cachePolicy;
@@ -68,16 +69,16 @@ typedef struct {
   return o;
 }
 
-+ (id) requestWithURL: (NSURL *)URL
++ (instancetype) requestWithURL: (NSURL *)URL
 {
   return [self requestWithURL: URL
 		  cachePolicy: NSURLRequestUseProtocolCachePolicy
 	      timeoutInterval: 60.0];
 }
 
-+ (id) requestWithURL: (NSURL *)URL
-	  cachePolicy: (NSURLRequestCachePolicy)cachePolicy
-      timeoutInterval: (NSTimeInterval)timeoutInterval
++ (instancetype) requestWithURL: (NSURL *)URL
+                    cachePolicy: (NSURLRequestCachePolicy)cachePolicy
+                timeoutInterval: (NSTimeInterval)timeoutInterval
 {
   NSURLRequest	*o = [[self class] allocWithZone: NSDefaultMallocZone()];
 
@@ -116,7 +117,8 @@ typedef struct {
 	  ASSIGN(inst->method, this->method);
 	  inst->shouldHandleCookies = this->shouldHandleCookies;
 	  inst->debug = this->debug;
-          inst->headers = [this->headers mutableCopy];
+	  inst->ioDelegate = this->ioDelegate;
+	  inst->headers = [this->headers mutableCopy];
 	}
     }
   return o;
@@ -177,16 +179,16 @@ typedef struct {
   return [self initWithURL: nil];
 }
 
-- (id) initWithURL: (NSURL *)URL
+- (instancetype) initWithURL: (NSURL *)URL
 {
   return [self initWithURL: URL
 	       cachePolicy: NSURLRequestUseProtocolCachePolicy
 	   timeoutInterval: 60.0];
 }
 
-- (id) initWithURL: (NSURL *)URL
-       cachePolicy: (NSURLRequestCachePolicy)cachePolicy
-   timeoutInterval: (NSTimeInterval)timeoutInterval
+- (instancetype) initWithURL: (NSURL *)URL
+                 cachePolicy: (NSURLRequestCachePolicy)cachePolicy
+             timeoutInterval: (NSTimeInterval)timeoutInterval
 {
   if ([URL isKindOfClass: [NSURL class]] == NO)
     {
@@ -284,6 +286,16 @@ typedef struct {
   return old;
 }
 
+- (id<GSLogDelegate>) setDebugLogDelegate: (id<GSLogDelegate>)d
+{
+  id<GSLogDelegate>     old = this->ioDelegate;
+
+  NSAssert(nil == d || [d conformsToProtocol: @protocol(GSLogDelegate)],
+    NSInvalidArgumentException);
+  this->ioDelegate = d;
+  return old;
+}
+
 - (NSTimeInterval) timeoutInterval
 {
   return this->timeoutInterval;
@@ -391,8 +403,8 @@ typedef struct {
 
       if ([value isKindOfClass: [NSString class]] == YES)
         {
-	  [self setValue: (NSString*)value forHTTPHeaderField: field];
-	}
+          [self setValue: (NSString*)value forHTTPHeaderField: field];
+        }
     }
 }
 
@@ -432,7 +444,14 @@ typedef struct {
     {
       this->headers = [_GSMutableInsensitiveDictionary new];
     }
-  [this->headers setObject: value forKey: field];
+  if (value != nil)
+    {
+      [this->headers setObject: value forKey: field];
+    }
+  else
+    {
+      [this->headers removeObjectForKey: field];
+    }
 }
 
 @end
@@ -444,6 +463,11 @@ typedef struct {
   return this->debug;
 }
 
+- (id<GSLogDelegate>) _debugLogDelegate
+{
+  return this->ioDelegate;
+}
+
 - (id) _propertyForKey: (NSString*)key
 {
   return [this->properties objectForKey: key];
