@@ -362,7 +362,16 @@ extern void ensure_oldNs(xmlNodePtr node);
   NSEnumerator	*enumerator = [attributes objectEnumerator];
   NSXMLNode	*attribute;
 
-  // FIXME: Remove all previous attributes
+  // Remove all previous attributes
+  NSArray *currentAttributes = [self attributes]; 
+  int index;
+  for (index = [currentAttributes count]-1; index >= 0; index--)
+    {
+	  NSXMLNode *attrNode = [currentAttributes objectAtIndex:index];
+      NSString *name = [attrNode name];
+	  [self removeAttributeForName:name];
+	}
+
   while ((attribute = [enumerator nextObject]) != nil)
     {
       [self addAttribute: attribute];
@@ -379,7 +388,16 @@ extern void ensure_oldNs(xmlNodePtr node);
   NSEnumerator	*en = [attributes keyEnumerator];
   NSString	*key;
 
-  // FIXME: Remove all previous attributes
+  // Remove all previous attributes
+  NSArray *currentAttributes = [self attributes]; 
+  int index;
+  for (index = [currentAttributes count]-1; index >= 0; index--)
+    {
+	  NSXMLNode *attrNode = [currentAttributes objectAtIndex:index];
+      NSString *name = [attrNode name];
+	  [self removeAttributeForName:name];
+	}
+
   while ((key = [en nextObject]) != nil)
     {
       NSString	*val = [[attributes objectForKey: key] stringValue];
