@@ -20,7 +20,9 @@
 
 #include <ctype.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <stdio.h>
 
 #import "GSPrivate.h"
@@ -39,6 +41,11 @@
 #define INT16SZ     2    /* for systems without 16-bit ints */
 #endif
 
+#ifndef WSAAPI
+#define WSAAPI
+#endif
+
+
 /*
  * WARNING: Don't even consider trying to compile this on a system where
  * sizeof(int) < 4.  sizeof(int) > 4 is fine; all the world's not a VAX.
@@ -56,11 +63,8 @@ static  __attribute__((unused)) const char *inet_ntop6(const u_char *src, char *
  *      Paul Vixie, 1996.
  */
 const char *
-inet_ntop(af, src, dst, size)
-        int af;
-        const void *src;
-        char *dst;
-        size_t size;
+WSAAPI
+inet_ntop(int af, const void *src, char *dst, size_t size)
 {
         switch (af) {
         case AF_INET:
