@@ -34,6 +34,7 @@
 #import "Foundation/NSHost.h"
 #import "Foundation/NSProcessInfo.h"
 #import "Foundation/NSByteOrder.h"
+#import "Foundation/NSURL.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
 
 #import "../GSPrivate.h"
@@ -950,6 +951,16 @@
     initToAddr: address port: port]);
   sock = socket(PF_INET, SOCK_STREAM, 0);
 
+  if (!ins)
+    {
+#if     defined(PF_INET6)
+      ins = AUTORELEASE([[GSInet6InputStream alloc]
+        initToAddr: address port: port]);
+      outs = AUTORELEASE([[GSInet6OutputStream alloc]
+        initToAddr: address port: port]);
+#endif
+    }
+
   /*
    * Windows only permits a single event to be associated with a socket
    * at any time, but the runloop system only allows an event handle to
@@ -1202,6 +1213,15 @@ done:
   return AUTORELEASE([[GSFileInputStream alloc] initWithFileAtPath: path]);
 }
 
++ (id) inputStreamWithURL: (NSURL *)url
+{
+  if ([url isFileURL])
+    {
+      return [self inputStreamWithFileAtPath: [url path]];
+    }
+  return [self inputStreamWithData: [url resourceDataUsingCache: YES]];
+}
+
 - (BOOL) getBuffer: (uint8_t **)buffer length: (NSUInteger *)len
 {
   [self subclassResponsibility: _cmd];
@@ -1226,6 +1246,17 @@ done:
   return [[GSFileInputStream alloc] initWithFileAtPath: path];
 }
 
+- (id) initWithURL: (NSURL *)url
+{
+  DESTROY(self);
+  if ([url isFileURL])
+    {
+      return [[GSFileInputStream alloc] initWithFileAtPath: [url path]];
+    }
+  return [[GSDataInputStream alloc]
+    initWithData: [url resourceDataUsingCache: YES]];
+}
+
 - (NSInteger) read: (uint8_t *)buffer maxLength: (NSUInteger)len
 {
   [self subclassResponsibility: _cmd];
