@@ -6,8 +6,15 @@
 #import <Foundation/NSProcessInfo.h>
 #import <Foundation/NSPathUtilities.h>
 #import <Foundation/NSError.h>
+#import <Foundation/NSThread.h>
 #import <Foundation/NSURL.h>
 
+#ifdef  EQ
+#undef  EQ
+#endif
+#define EPSILON (FLT_EPSILON*100)
+#define EQ(x,y) ((x >= y - EPSILON) && (x <= y + EPSILON))
+
 @interface      MyHandler : NSObject
 {
   @public
@@ -46,18 +53,20 @@
 
 int main()
 {
-  NSAutoreleasePool   *arp = [NSAutoreleasePool new];
-  NSFileManager *mgr = [NSFileManager defaultManager];
-  NSString *dir = @"NSFileManagerTestDir"; 
-  MyHandler *handler = [MyHandler new];
-  NSDictionary *attr;
-  NSString *dirInDir;
-  NSString *str1,*str2;
-  NSString *tmp;
-  NSError *err;
-  NSDictionary *errInfo;
-  BOOL exists;
-  BOOL isDir;
+  NSAutoreleasePool   	*arp = [NSAutoreleasePool new];
+  NSFileManager		*mgr = [NSFileManager defaultManager];
+  NSString 		*dir = @"NSFileManagerTestDir"; 
+  MyHandler 		*handler = [MyHandler new];
+  NSDictionary 		*attr;
+  NSString 		*dirInDir;
+  NSString 		*str1;
+  NSString 		*str2;
+  NSMutableString	*mstr;
+  NSString 		*tmp;
+  NSError 		*err;
+  NSDictionary 		*errInfo;
+  BOOL 			exists;
+  BOOL 			isDir;
 
   dirInDir = [dir stringByAppendingPathComponent: @"WithinDirectory"];
 
@@ -120,6 +129,19 @@ int main()
          "NSFileManager can get current dir");
   }
   
+  PASS([mgr createFileAtPath: @"NSFMFile" 
+                    contents: [NSData data]
+		  attributes: nil],
+       "NSFileManager creates an empty file")
+  PASS([mgr fileExistsAtPath: @"NSFMFile"],"-fileExistsAtPath: agrees")
+  str1 = [NSString stringWithContentsOfFile: @"NSFMFile"];
+  PASS_EQUAL(str1, @"", "empty file produces empty string")
+  mstr = [NSMutableString stringWithContentsOfFile: @"NSFMFile"];
+  PASS([mstr isKindOfClass: [NSMutableString class]] && [mstr isEqual: @""],
+    "empty file produces empty mutable string")
+  PASS([mgr removeFileAtPath: @"NSFMFile" handler: handler], 
+    "NSFileManager removes an empty file")
+
   str1 = @"A string";
   PASS([mgr createFileAtPath: @"NSFMFile" 
                     contents: [str1 dataUsingEncoding: 1]
@@ -135,12 +157,37 @@ int main()
       "-contentsOfDirectoryAtPath: agrees");
   }
 
+  {
+    NSArray	*a;
+    NSString *dir1 = [mgr currentDirectoryPath];
+
+    [mgr changeCurrentDirectoryPath: [dir1 stringByDeletingLastPathComponent]];
+    a = [mgr contentsOfDirectoryAtPath: dir error: 0];
+    PASS(1 == [a count] && [[a lastObject] isEqual: @"NSFMFile"],
+      "-contentsOfDirectoryAtPath: agrees with different current directory");
+    [mgr changeCurrentDirectoryPath: dir1];
+  }
+
+  {
+    NSArray	*a;
+    NSString *dir1 = [mgr currentDirectoryPath];
+    NSURL *dirURL = [NSURL fileURLWithPath: dir1];
+    NSURL *fileURL = [dirURL URLByAppendingPathComponent: @"NSFMFile"];
+
+    [mgr changeCurrentDirectoryPath: [dir1 stringByDeletingLastPathComponent]];
+    a = [mgr contentsOfDirectoryAtURL: dirURL includingPropertiesForKeys: nil
+                              options: 0 error: NULL];
+    PASS(1 == [a count] && [[a lastObject] isEqual: fileURL],
+      "-contentsOfDirectoryAtURL: agrees with different current directory");
+    [mgr changeCurrentDirectoryPath: dir];
+  }
+
   {
     NSData *dat1 = [mgr contentsAtPath: @"NSFMFile"];
     str2 = [[NSString alloc] initWithData: dat1 encoding: 1];
     PASS([str1 isEqualToString: str2], "NSFileManager file contents match");
   }
-  
+  [NSThread sleepForTimeInterval: 1.0]; // So date of file is clearly in past
   [handler reset];
   PASS([mgr copyPath: @"NSFMFile"
               toPath: @"NSFMCopy"
@@ -152,7 +199,39 @@ int main()
     str2 = [[NSString alloc] initWithData: dat1 encoding: 1];
     PASS([str1 isEqual: str2],"NSFileManager copied file contents match");
   }
-  
+  NSDictionary *oa = [mgr fileAttributesAtPath: @"NSFMFile" traverseLink: NO];
+  NSDictionary *na = [mgr fileAttributesAtPath: @"NSFMCopy" traverseLink: NO];
+  NSTimeInterval        ot, nt;
+  ot = [[oa fileCreationDate] timeIntervalSinceReferenceDate];
+  nt = [[na fileCreationDate] timeIntervalSinceReferenceDate];
+  PASS(EQ(ot, nt), "copy creation date equals original")
+  ot = [[oa fileModificationDate] timeIntervalSinceReferenceDate];
+  nt = [[na fileModificationDate] timeIntervalSinceReferenceDate];
+  PASS(EQ(ot, nt), "copy modification date equals original")
+  {
+    NSData *dat1 = [mgr contentsAtPath: @"NSFMFile"];
+    NSError *err;
+    BOOL ok;
+
+    ok = [dat1 writeToFile: @"NSFMFile"
+                   options: NSDataWritingAtomic
+                     error: &err];
+    PASS(ok, "can rewrite data file")
+    if (NO == ok) NSLog(@"Problem: %@ with %@", err, dat1);
+    na = [mgr fileAttributesAtPath: @"NSFMFile" traverseLink: NO];
+#if !defined(_WIN32)
+    /* Atomic copy to an existing file on windows retains the file
+     * creation date, but on unix a rename produces a new timestamp.
+     */
+    ot = [[oa fileCreationDate] timeIntervalSinceReferenceDate];
+    nt = [[na fileCreationDate] timeIntervalSinceReferenceDate];
+    PASS(!EQ(ot, nt), "rewritten file creation date changed")
+#endif
+    ot = [[oa fileModificationDate] timeIntervalSinceReferenceDate];
+    nt = [[na fileModificationDate] timeIntervalSinceReferenceDate];
+    PASS(!EQ(ot, nt), "rewritten file modification date changed")
+  }
+
   PASS([mgr movePath: @"NSFMFile"
               toPath: @"NSFMMove"
 	     handler: handler],
@@ -212,6 +291,28 @@ int main()
   PASS([mgr createDirectoryAtPath: @"subdir" attributes: nil],
        "NSFileManager can create a subdirectory");
   
+  {
+    NSURL			*u;
+    NSDirectoryEnumerator	*e;
+    unsigned			found = 0;
+
+    e = [mgr enumeratorAtURL: [NSURL fileURLWithPath: @"."]
+  includingPropertiesForKeys: nil
+		     options: 0
+		errorHandler: nil]; 
+
+    while (nil != (u = [e nextObject]))
+      {
+	NSString	*c = [[u path] lastPathComponent];
+
+	if ([c isEqualToString: @"NSFMCopy"])
+	  found++;
+	if ([c isEqualToString: @"subdir"])
+	  found++;
+      }
+    PASS(2 == found, "URL enumerator finds expected file and subdirectory")
+  }
+
   PASS([mgr changeCurrentDirectoryPath: @"subdir"], 
        "NSFileManager can move into subdir");
 
@@ -274,8 +375,6 @@ int main()
       dir = [dir stringByStandardizingPath];
       PASS([mgr removeFileAtPath: dir handler: handler], "removed directory");
       PASS(![mgr fileExistsAtPath: dir], "directory no longer exists");
-GSPrintf(stdout, @"%@\n", dir);
-GSPrintf(stderr, @"%@\n", dir);
     }
   
   err = nil;
