@@ -10,6 +10,8 @@ int main()
   NSAutoreleasePool   *arp = [NSAutoreleasePool new];
   NSFileManager *mgr;
   NSString *helpers;
+  NSString *testecho;
+  NSString *testcat;
   NSArray *args;
   id task;
   id info;
@@ -18,6 +20,16 @@ int main()
   id pth2;
   BOOL yes;
 
+  /* Windows MSVC adds the '.exe' suffix to executables
+   */
+#if defined(_MSC_VER)
+  testecho = @"testecho.exe";
+  testcat = @"testcat.exe";
+#else
+  testecho = @"testecho";
+  testcat = @"testcat";
+#endif
+  
   info = [NSProcessInfo processInfo];
   env = [[info environment] mutableCopy];
   yes = YES;
@@ -32,7 +44,8 @@ int main()
   helpers = [helpers stringByAppendingPathComponent: @"Helpers"];
   helpers = [helpers stringByAppendingPathComponent: @"obj"];
 
-  pth1 = [helpers stringByAppendingPathComponent: @"testcat"];
+  pth1 = [helpers stringByAppendingPathComponent: testcat];
+  pth2 = [helpers stringByAppendingPathComponent: testecho];
 
   /* Try some tasks.  Make sure the program we use is common between Unix
      and Windows (and others?) */
@@ -43,7 +56,6 @@ int main()
 
   task = [NSTask new];
   args = [NSArray arrayWithObjects: @"xxx", @"yyy", nil];
-  pth2 = [helpers stringByAppendingPathComponent: @"testecho"];
   [task setEnvironment: env];
   [task setLaunchPath: pth2];
   [task setArguments: args];
