@@ -14,18 +14,6 @@ int main(int argc, char **argv, char **env)
   BOOL loaded;
   NSString *helperPath;
   
-  /* The following test cases depend on the GSInetServerStream
-   * class which is completely broken on Windows.
-   *
-   * See: https://github.com/gnustep/libs-base/issues/266
-   *
-   * We will mark the test cases as hopeful on Windows.
-   */
-#if defined(_WIN32)
-  NSLog(@"Marking local web server tests as hopeful because GSInetServerStream is broken on Windows");
-  testHopeful = YES;
-#endif
-
   // load the test suite's classes
   fm = [NSFileManager defaultManager];
   helperPath = [[fm currentDirectoryPath]
@@ -37,7 +25,7 @@ int main(int argc, char **argv, char **env)
     {
       Class testClass;
       TestWebServer *server;
-      BOOL debug = NO;
+      BOOL debug = GSDebugSet(@"dflt");
       NSURL *url;
       NSError *error = nil;
       NSURLRequest *request;
@@ -49,16 +37,20 @@ int main(int argc, char **argv, char **env)
       // create a shared TestWebServer instance for performance
       // by default it requires the basic authentication with the pair
       // login:password
-      server = [[testClass testWebServerClass] new];
+      server = [[[testClass testWebServerClass] alloc]
+        initWithAddress: @"localhost"
+                   port: @"1230"
+                   mode: NO
+                  extra: nil];
       [server setDebug: debug];
-      [server start: nil]; // localhost:1234 HTTP
+      [server start: nil]; // localhost:1230 HTTP
 
       /*
        *  Simple GET via HTTP with some response's body and
        *  the response's status code 200
        */
       url = [NSURL
-        URLWithString: @"http://login:password@localhost:1234/index"];
+        URLWithString: @"http://login:password@localhost:1230/index"];
       request = [NSURLRequest requestWithURL: url];
       data = [NSURLConnection sendSynchronousRequest: request
 				   returningResponse: &response
@@ -77,10 +69,6 @@ int main(int argc, char **argv, char **env)
 		  format: @"can't load bundle TestConnection"];
     }
 
-#if defined(_WIN32)
-  testHopeful = NO;
-#endif
-
   DESTROY(arp);
 
   return 0;
