@@ -22,11 +22,7 @@
 
 #import "common.h"
 
-#import "Foundation/NSAutoreleasePool.h"
-#import "Foundation/NSArray.h"
-#import "Foundation/NSDictionary.h"
-#import "Foundation/NSSet.h"
-#import "Foundation/NSUserDefaults.h"
+#import "Foundation/Foundation.h"
 #import "AGSHtml.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
 #import "GNUstepBase/NSMutableString+GNUstepBase.h"
@@ -108,6 +104,7 @@ static NSString		*mainFont = nil;
   RELEASE(localRefs);
   RELEASE(projectRefs);
   RELEASE(indent);
+  RELEASE(fileName);
   DEALLOC
 }
 
@@ -128,9 +125,15 @@ static NSString		*mainFont = nil;
 
 - (id) init
 {
-  indent = [[NSMutableString alloc] initWithCapacity: 64];
-  project = RETAIN([[NSUserDefaults standardUserDefaults]
-    stringForKey: @"Project"]);
+  if (nil != (self = [super init]))
+    {
+      NSUserDefaults	*defs = [NSUserDefaults standardUserDefaults];
+
+      indent = [[NSMutableString alloc] initWithCapacity: 64];
+      project = RETAIN([defs stringForKey: @"Project"]);
+      verbose = [defs boolForKey: @"Verbose"];
+      warn = [defs boolForKey: @"Warn"];
+    }
   return self;
 }
 
@@ -281,10 +284,11 @@ static NSString		*mainFont = nil;
   return [s stringByReplacingString: @":" withString: @"$"];
 }
 
-- (NSString*) outputDocument: (GSXMLNode*)node
+- (NSString*) outputDocument: (GSXMLNode*)node name: (NSString*)file
 {
   NSMutableString	*buf;
 
+  ASSIGN(fileName, file);
   if (localRefs == nil)
     {
       localRefs = [AGSIndex new];
@@ -305,6 +309,7 @@ static NSString		*mainFont = nil;
   [self decIndent];
   [buf appendString: @"</html>\n"];
 
+  DESTROY(fileName);
   return buf;
 }
 
@@ -1193,6 +1198,15 @@ static NSString		*mainFont = nil;
 		  GSXMLNode		*email = nil;
 		  GSXMLNode		*url = nil;
 		  GSXMLNode		*desc = nil;
+		  NSString		*name;
+
+		  name = [[author attributes] objectForKey: @"name"];
+		  name = [name stringByTrimmingSpaces];
+		  if ([name length] == 0)
+		    {
+		      name = [NSString stringWithFormat: @"Generated by %@",
+			NSFullUserName()];
+		    }
 
 		  children = [children nextElement];
 
@@ -1216,8 +1230,7 @@ static NSString		*mainFont = nil;
 		  if (url == nil)
 		    {
 		      [buf appendString: @"<dt>"];
-		      [buf appendString: [[[author attributes]
-			objectForKey: @"name"] stringByEscapingXML]];
+		      [buf appendString: [name stringByEscapingXML]];
 		    }
 		  else
 		    {
@@ -1225,8 +1238,7 @@ static NSString		*mainFont = nil;
 		      [buf appendString: [[url attributes]
 			objectForKey: @"url"]];
 		      [buf appendString: @"\">"];
-		      [buf appendString: [[[author attributes]
-			objectForKey: @"name"] stringByEscapingXML]];
+		      [buf appendString: [name stringByEscapingXML]];
 		      [buf appendString: @"</a>"];
 		    }
 		  if (email != nil)
@@ -1267,9 +1279,24 @@ static NSString		*mainFont = nil;
 	    }
 	  if ([[children name] isEqual: @"date"] == YES)
 	    {
+	      GSXMLNode	*tmp = [children firstChild];
+	      NSString	*str;
+
 	      [buf appendString: indent];
 	      [buf appendString: @"<p><b>Date:</b> "];
-	      [self outputText: [children firstChild] to: buf];
+	      if (nil == tmp
+		|| ([tmp type] == XML_TEXT_NODE
+		  && [(str = [[tmp escapedContent] stringByTrimmingSpaces])
+		    length] == 0))
+		{
+            	  str = [NSString stringWithFormat: @"Generated at %@",
+		    [NSDate date]];
+		  [buf appendString: str];
+		}
+	      else
+		{
+	          [self outputText: tmp to: buf];
+		}
 	      [buf appendString: @"</p>\n"];
 	      children = [children nextElement];
 	    }
@@ -1719,15 +1746,24 @@ static NSString		*mainFont = nil;
 	    }
 	  if (s == nil)
 	    {
-	      if (c == nil)
+	      if (warn)
 		{
-		  NSLog(@"Location of %@ '%@' (referenced from %@) "
-		    @"not found or not unique.", type, r, base);
-		}
-	      else
-		{
-		  NSLog(@"Location of the %@ version of %@ '%@' (referenced "
-		    @"from %@) not found.", c, type, r, base);
+		  NSString	*ref;
+
+		  ref = [NSString stringWithFormat:
+		    @" (referenced from %@ in %@).",
+		    base, fileName];
+		  if (c == nil)
+		    {
+		      NSLog(@"Warning - location of %@ '%@'"
+			@" not found or not unique %@.",
+			type, r, ref);
+		    }
+		  else
+		    {
+		      NSLog(@"Warning - location of the %@ version of %@ '%@'"
+			@" not found %@.", c, type, r, ref);
+		    }
 		}
 	      if (tmp == nil)
 		{
