@@ -42,6 +42,8 @@
   unsigned		ssect;
   unsigned		sssect;
 }
+- (NSDictionary*) authors;
+- (NSArray*) find: (NSString*)key;
 - (NSString*) globalRef: (NSString*)ref type: (NSString*)type;
 - (void) makeRefs: (GSXMLNode*)node;
 - (void) mergeRefs: (NSDictionary*)more override: (BOOL)flag;
@@ -49,6 +51,7 @@
 - (NSArray*) methodsInUnit: (NSString*)aUnit;
 - (NSMutableDictionary*) refs;
 - (void) setDirectory: (NSString*)path;
+- (void) setEmail: (NSString*)address forAuthor: (NSString*)name;
 - (void) setGlobalRef: (NSString*)ref type: (NSString*)type;
 - (void) setOutputs: (NSArray*)a forHeader: (NSString*)h;
 - (void) setRelationship: (NSString*)r from: (NSString*)from to: (NSString*)to;
