@@ -58,7 +58,7 @@ static BOOL snuggleEnd(NSString *t)
   if (set == nil)
     {
       set = [NSCharacterSet characterSetWithCharactersInString: @"]}).,;"];
-      IF_NO_GC([set retain];)
+      IF_NO_ARC([set retain];)
     }
   return [set characterIsMember: [t characterAtIndex: 0]];
 }
@@ -70,7 +70,7 @@ static BOOL snuggleStart(NSString *t)
   if (set == nil)
     {
       set = [NSCharacterSet characterSetWithCharactersInString: @"[{("];
-      IF_NO_GC([set retain];)
+      IF_NO_ARC([set retain];)
     }
   return [set characterIsMember: [t characterAtIndex: [t length] - 1]];
 }
@@ -91,8 +91,8 @@ static BOOL snuggleStart(NSString *t)
  * And finally, here is the actual class description ... outside the chapter.
  * This is the class description for <code>AGSOutput</code>, including some
  * sample uses of GSDoc, such as cross-references (see [NSString]).
- * Functions, like escapeType(), are automatically referenced (if they are
- * found).
+ * Functions, (recognised as an identifier immediately followed by open and
+ * closing brackets), are automatically referenced (if they are found).
  */
 @implementation	AGSOutput
 
@@ -397,7 +397,7 @@ static BOOL snuggleStart(NSString *t)
   authors = [info objectForKey: @"authors"];
   if (authors == nil)
     {
-      tmp = [NSString stringWithFormat: @"Generated by %@", NSUserName()];
+      tmp = [NSString stringWithFormat: @"Generated by %@", NSFullUserName()];
       [str appendString: @"    <author name=\""];
       [str appendString: tmp];
       [str appendString: @"\"></author>\n"];
@@ -746,7 +746,7 @@ static BOOL snuggleStart(NSString *t)
 
   if (warn == YES && [[d objectForKey: @"Implemented"] isEqual: @"YES"] == NO)
     {
-      NSLog(@"Warning ... %@ %@ is not implemented where expected", kind, name);
+      NSLog(@"Warning - %@ %@ is not implemented where expected", kind, name);
     }
 
   [str appendFormat: @"      <%@ type=\"", kind];
@@ -790,7 +790,7 @@ static BOOL snuggleStart(NSString *t)
 
   if (warn == YES && [[d objectForKey: @"Implemented"] isEqual: @"YES"] == NO)
     {
-      NSLog(@"Warning ... function %@ is not implemented where expected", name);
+      NSLog(@"Warning - function %@ is not implemented where expected", name);
     }
 
   /**
@@ -1081,7 +1081,7 @@ static BOOL snuggleStart(NSString *t)
 	} while (r.length > 0);
       if (m != nil)
 	{
-	  IF_NO_GC([m autorelease];)
+	  IF_NO_ARC([m autorelease];)
 	}
     }
 
@@ -1172,7 +1172,7 @@ static BOOL snuggleStart(NSString *t)
 	}
       else if (warn == YES)
 	{
-	  NSLog(@"Warning ... unit %@ is not implemented where expected", name);
+	  NSLog(@"Warning - unit %@ is not implemented where expected", name);
 	}
     }
   else
@@ -2365,7 +2365,8 @@ static BOOL snuggleStart(NSString *t)
 	  [str appendString: @"    <title>"];
 	  [str appendString: kind];
 	  [str appendString: @"</title>\n"];
-	  tmp = [NSString stringWithFormat: @"Generated by %@", NSUserName()];
+	  tmp = [NSString stringWithFormat: @"Generated by %@",
+	    NSFullUserName()];
 	  [str appendString: @"    <author name=\""];
 	  [str appendString: tmp];
 	  [str appendString: @"\"></author>\n"];
